<?php

namespace Teknisa\FIN\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\Libs\Service\CRUD;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIN\Services\AutomacaoFinanceiraService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Remote\FileUploader\Server;

class AutomacaoFinanceiraController { 
    
    protected $crudService; 
    protected $parameterBag; 
    
    /**
     * @param CRUD                       $crudService
     * @param ParameterBag               $parameterBag
     * @param EntityManager              $entityManager
     * @param AutomacaoFinanceiraService $service
     * @param                            $connectionCustom
     * @param                            $fileUploader
     * @param FileServer                 $fileServerLink
     * 
     */
    public function __construct(CRUD $crudService, ParameterBag $parameterBag, EntityManager $entityManager, AutomacaoFinanceiraService $service, $connectionCustom, Server $fileUploader, $fileServerLink) {
        $this->crudService      = $crudService;
        $this->parameterBag     = $parameterBag;
        $this->entityManager    = $entityManager;
        $this->service          = $service;
        $this->connectionCustom = $connectionCustom;
        $this->fileUploader     = $fileUploader;
        $this->fileServerLink   = $fileServerLink;
    }
    
    public function execEnvioBoletoEmitido(DTO\Request $request, DTO\Response $response) {
       try {
            // Busca Parâmetros
            $row = $request->getParameters();
            
            // Variáveis Iniciais
            $nrorg = $row['NRORG'];
            $newEntityManager = null;
            
            // Seta Variáveis na Sessão
            $_SESSION['_sf2_attributes']['NRORG'] = $nrorg;
            $_SESSION['_sf2_attributes']['ORGANIZATION_ID'] = $nrorg;
            
            // Conectação Banco de dados
            $useVpd = InstanceManager::getInstance()->getService('EntityManager')->getConnection()->getParams()['useVpd'];
            
            // Configuração VPN
            $this->setVPD($nrorg, $useVpd, $newEntityManager);
            
            // Executa Envio de Boleto Emitido
            $this->service->executeEmissao($nrorg, $useVpd, $newEntityManager, $this->fileUploader, $request, $response);
            
        } catch(\Exception $e) {
            var_dump($e->getMessage());
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function execEnvioCartaCobranca(DTO\Request $request, DTO\Response $response) {
       try {
            // Busca Parâmetros
            $row = $request->getParameters();
            
            // Variáveis Iniciais
            $nrorg = $row['NRORG'];
            $newEntityManager = null;
            
            // Seta Variáveis na Sessão
            $_SESSION['_sf2_attributes']['NRORG'] = $nrorg;
            $_SESSION['_sf2_attributes']['ORGANIZATION_ID'] = $nrorg;
            
            // Conectação Banco de dados
            $useVpd = InstanceManager::getInstance()->getService('EntityManager')->getConnection()->getParams()['useVpd'];
            
            // Configuração VPN
            $this->setVPD($nrorg, $useVpd, $newEntityManager);
            
            // Executa Envio de Boleto Emitido
            $this->service->executeCobranca($nrorg, $useVpd, $newEntityManager, $this->fileUploader, $request, $response);
            
        } catch(\Exception $e) {
            var_dump($e->getMessage());
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function setVPD($nrorg, $useVpd, &$newEntityManager) {
        if($useVpd) {
            $newEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
            
            if(empty($newEntityManager)) throw new \Exception('Falha ao gerar o newEntityManager.');
        }
        
        if($nrorg > 1) $this->setFileServer($nrorg);
    }
    
    private function setFileServer($nrorg) {
        $httpInterface = new cURLRequest($this->fileServerLink);
        $objFileUploaderAPIKey = InstanceManager::getInstance()->getService('fileUploaderAPIKeyProvider');
        $apiKey = (array) $objFileUploaderAPIKey;
        $hashApiKey = '';
        
        foreach($apiKey as $key => $conteudo) {
            if(strpos($key, 'apiKeyMapping') !== false) {
                if(array_key_exists($nrorg, $conteudo)) {
                    $hashApiKey = $conteudo[$nrorg];
                    $this->fileUploader = new Server($httpInterface, $hashApiKey , []);
                    break;
                }
                else {
                    throw new \Exception('Fileserver nao esta configurado corretamente para o NRORG: '.$nrorg);
                    break;
                }
            }
            else break;
        }
    }

    

}