<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Exception;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Fiscal\Entities\Notafiscal;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabBtgService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabBtgService extends GeraCnabService{
    
    const NOME_BANCO = 'BTG Pactual';
    const NUM_LOTE_HEADER_ARQUIVO = '0000';
    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_ARQUIVO_HEADER_LOTE = '1';
    const ID_TP_ARQUIVO_SEGMENTO = '3';
    const ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    const TIPO_INS_JUR_CPF = 1;
    const TIPO_INS_JUR_CNPJ = 2;
    const CDBANCO = '208';
    const REMESSA = 1;
    const RETORNO = 2;
    const VERSAO_LAYOUT = '103';
    const TIPO_OPERACAO_CREDITO = 'C';
    const TIPO_SERVICO = '20';
    const TIPO_SERVICO_TRIBUTO = '22';
    const VERSAO_LOTE = '046';
    const VERSAO_LOTE_TITULO_BANC = '030';
    const VERSAO_LOTE_TITULO_TRIBUTO = '010';
    const SEGMENTO_A = 'A';
    const EXCLUSAO = 'E';
    const INCLUSAO = 'I';
    const BLOQUEIO = 'B';
    const MOEDA = 'BRL';
    const SEGMETO_B = 'B';
    const LOTE_SERVICO_TRAILER_ARQUIVO = '9999';
    const ID_TP_ARQUIVO_TRAILER = '9';
    const SEGMENTO_J = 'J';
    const SEGMENTO_J52 = '52';
    const SEGMENTO_O = 'O';
    const SEGMETO_I = 'I';
    const SEGMENTO_N = 'N';
    const TIPO_BAIXA_PIX = 'F';
    const TIPO_BAIXA_PIX_QRCODE = 'Z';
    const TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL = 'N';
    const TIPO_BAIXA_PAG_IPVA = 'V';
    const TIPO_BAIXA_PAG_GARE = 'X';
    const TIPO_BAIXA_PAG_DARJ = 'J';

    private $num_header_lote = 0;
    private $seqRegLote = 0;
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $vrPagtLote = 0.00; // Somatório dos valores de inclusao do lote;
    private $formapgto;

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->filial->getNrinsjurfili();
                $this->nomeempresa = $this->filial->getNmfilial();
                $this->nomerazempresa = $this->filial->getNmrazsocfili();
                $this->dsEndeEF = $this->filial->getEndefiliprinc()->getDsendefili();
                $this->muniEF = $this->filial->getEndefiliprinc()->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->filial->getEndefiliprinc()->getNrcepfili();
                $this->estadoEF = $this->filial->getEndefiliprinc()->getSgestado();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
                $this->nomeempresa = $this->empresa->getNmempresa();
                $this->nomerazempresa = $this->empresa->getNmrazsocempr();
                $this->dsEndeEF = $this->empresa->getDsendeempr();
                $this->muniEF = $this->empresa->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->empresa->getNrcepempr();
                $this->estadoEF = $this->empresa->getSgestado();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{

            $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);

            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                //Gera Segmentos AB
                if($segmento == 'SEGMENTOAB'){

                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);

                        foreach($boletosBco as $boleto){
                            if($this->remessa->getIdprenegociado() == 'S'){
                                $segmentoI = self::geraDetalheSegmentoI($boleto);
                                fprintf($file, "%s",$segmentoI);
                            }
                            $segmentoA = self::geraDetalheSegmentoA($boleto, $boleto->getCdbancbolp());
                            $segmentoB = self::geraDetalheSegmentoB($boleto);
                            fprintf($file, "%s",$segmentoA);
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                // Fim Segmento AB

                //Gera Segmentos J
                if($segmento == 'SEGMENTOJ'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLoteBoletoBanc($key);
                        fprintf($file, "%s",$headerlote);
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ($boleto);
                            fprintf($file, "%s",$segmentoJ);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ52($boleto);
                            fprintf($file, "%s",$segmentoJ);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } // Fim Segmento J

                //Gera Segmentos O
                if($segmento == 'SEGMENTOO'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLoteTributos($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                            /** @var $boleto Boletpagar */
                            $segmentoO = self::geraSegmentoO($boleto);
                            fprintf($file, "%s",$segmentoO);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } // Fim Segmento O
                
                
                //Gera Segmentos N
                if($segmento == 'SEGMENTON'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLoteTributos($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                            /** @var $boleto Boletpagar */
                            $segmentoN = self::geraSegmentoN($boleto);
                            fprintf($file, "%s",$segmentoN);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } // Fim Segmento N
            }
            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /** Retorna array de Boletos Agrupados por Forma de Pagamento e por código do banco favorecido*/
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentosAB = array(Tipobaixapag::TIPOBAIXA_CNAB,
            Tipobaixapag::TIPOBAIXA_CNAB_DOC,
            Tipobaixapag::TIPOBAIXA_TED_CIP,
            Tipobaixapag::TIPO_BAIXA_TED_STR,
            self::TIPO_BAIXA_PIX,
            self::TIPO_BAIXA_PIX_QRCODE
        );

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

        $segmentoO = array(
            Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, 
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, 
            Tipobaixapag::TIPO_BAIXA_FGTS,
            self::TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL
        );
        
        $segmentoN = array(
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES,
            Tipobaixapag::TIPO_BAIXA_GPS,
            self::TIPO_BAIXA_PAG_IPVA,
            self::TIPO_BAIXA_PAG_GARE,
            self::TIPO_BAIXA_PAG_DARJ
        );

        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){

            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag(
                $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
            );

            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());

            if($tipobaixapag)
                $idbaixautilp = $tipobaixapag->getIdbaixautilp();
            else $idbaixautilp = null;

            if(in_array($idbaixautilp,$segmentosAB)){

                if(self::getBanco($boleto) == GeraCnabService::CODIGO_COMPENSACAO_BTG)
                    $boletosAgroup['SEGMENTOAB'] [GeraCnabService::CODIGO_COMPENSACAO_BTG] [] = $boleto;
                else
                    $boletosAgroup['SEGMENTOAB'] ['OUTROS_BANCOS'] [] = $boleto;

            }else{
                if(in_array($idbaixautilp,$segmentoJ)){
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BTG)){
                        $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_BTG] [] = $boleto;
                    }
                    else{
                        $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                    }

                }else{
                    if(in_array($idbaixautilp,$segmentoO)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BTG)){
                            $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_BTG] [] = $boleto;
                        }
                        else{
                            $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;
                        }

                    }else{
                        if(in_array($idbaixautilp,$segmentoN)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BTG)){
                            $boletosAgroup['SEGMENTON'] [GeraCnabService::CODIGO_COMPENSACAO_BTG] [] = $boleto;
                        }
                        else{
                            $boletosAgroup['SEGMENTON'] ['OUTROS_BANCOS'] [] = $boleto;
                        }

                        }else{
                            throw new \Exception ('Operação bloqueada. Tipo de Baixa não reconhecido.');
                        }
                    }
                }
            }
        }
        return $boletosAgroup;
    }


    // Registro Header de Arquivo
    /**
     * @throws Exception
     * @return string
     */
    public function geraHeaderArquivo(){

        try {

            $this->qtdRegiArq = 0;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= self::NUM_LOTE_HEADER_ARQUIVO;

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;

            //CNAB - Posição: 9 a 17 - Picture: X(9)
            $linha.= self::preenche_brancos('',9);

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco - Posição: 33 a 52 - Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Agência Mantenedora da Conta - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
                
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da Conta Corrente - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //Dígito Verificador da Ag/Conta - Posição: 72 a 72 - Picture: X(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
            $nmbanco = self::NOME_BANCO;
            $linha.= self::preenche_brancos($nmbanco,30);


            //CNAB - Posição: 133 a 142 - Picture: X(10)
            $linha.= self::preenche_brancos('',10);

            //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
            $linha.= self::REMESSA;

            //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
            $linha.= self::getDataGeracao();

            //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
            $linha.= self::getHorarioAtual();

            //Número de sequência do arquivo - Posição: 158 a 163 - Picture: 9(06)
            $linha.= substr($this->remessa->getNrseqremessapag(),-6);

            //Número da versão do layout do arquivo - Posição: 164 a 166 - Picture: 9(03)
            $linha.= self::VERSAO_LAYOUT;

            //Densidade de gravação do arquivo - Posição: 167 a 171 - Picture: 9(5)
            $linha.= str_repeat('0',5);

            //Campo para uso exclusivo do banco - Posição: 172 a 191 - Picture: X(20)
            //Campo para uso exclusivo da empresa - Posição: 192 a 211 - Picture: X(20)
            //CNAB - Posição: 212 a 240 - Picture X(29)
            $linha.= str_repeat(' ',69);

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, $boletos = null){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Lote de Serviço - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            self::formaPagamento($cdbanco, $boletos);
            $linha.= $this->formapgto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE;

            //CNAB - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Agência Mantenedora da Conta - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
            
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //Dígito Verificador da Ag/Conta  - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Mensagem - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ',40);
            
            //Logradouro - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($this->dsEndeEF, 0, 30)), 30,' ',STR_PAD_RIGHT);
            
            //Número - Posição: 173 a 177 - Picture:9(5)
            $linha.= str_repeat('0',5);
            
            //Complemento do Endereço - Posição: 178 a 192 - Picture: X(015) 
            $linha.= str_repeat(' ',15);
            
            //Cidade - Posição: 193 a 212 - Picture: X(20) 
            $linha.= str_pad(Util::campoCnabValido(substr($this->muniEF, 0, 20)), 20,' ',STR_PAD_RIGHT);
            
            //Cep - Posição:213 a 217 - Picture:9(5)
            $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 5)), 5,' ',STR_PAD_RIGHT);
            
            //Complemento do CEP - Posição: 218 a 220 - Picture: 9(3)
            $linha.= str_repeat('0',3);
            
            //Estado - Posição: 221 a 222 - Picture: X(2)
            $linha.= str_pad(Util::campoCnabValido($this->estadoEF), 2,' ',STR_PAD_RIGHT);
            
            //Indicativo de Forma de Pagamento - - Posição: 223 a 224 - Picture: 9(2)
            $linha.= str_repeat('0',2);
            
            //CNAB - Posição: 225 a 230 - Picture: X(6)
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',16);

            return $linha."\r\n";
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLoteBoletoBanc($cdbanco){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            $this->formapgto = '';
            if($cdbanco == self::CODIGO_COMPENSACAO_BTG)
                $this->formapgto = '30';
            else
                $this->formapgto = '31';

            $linha.= $this->formapgto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE_TITULO_BANC;

            //CNAB - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //Número de Inscrição da Empresa - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco - Posição: 33 a 52 - Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Agência Mantenedora da Conta - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }

            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da Conta Corrente - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            // Dígito Verificador da Ag/Conta - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Mensagem - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ',40);

            //Endereço - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($this->dsEndeEF, 0, 30)), 30,' ',STR_PAD_RIGHT);

            //Número do Local - Posição: 173 a 177 - Picture: 9(5)
            $linha.= str_repeat('0',5);

            //Complemento Endereço - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //CIDADE - Posição: 193 a 212 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //CEP e Complemento do CEP - Posição: 213 a 220 - Picture: X(8)
            $linha.= str_repeat('0',8);

            //Sigla do Estado - Posição: 221 a 222 - Picture: X(2)
            $linha.= $this->estadoEF;
            
            //CNAB - Posição: 223 a 230 - Picture: X(8)
            $linha.= str_repeat(' ',8);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }


    public function geraHeaderLoteTributos($cdbanco, $boletos = null){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $linha.= self::TIPO_SERVICO_TRIBUTO;

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            self::formaPagamento($cdbanco, $boletos);
            $linha.= $this->formapgto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= self::VERSAO_LOTE_TITULO_TRIBUTO;

            //CNAB - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            if ($this->remessa->getCdfilial()){
                if ($this->filial->getIdtpijurfili() == 'F'){
                    $linha .= '1';
                }else{
                    $linha .= '2';
                }
                $nrinsjurfili = trim($this->filial->getNrinsjurfili());
                if (strlen($nrinsjurfili) > 14){
                    $msg = "Código da inscrição da filial deve ter no máximo 14 posições.";
                }
                $linha .= $nrinsjurfili;
                $nmemprfili = str_pad($this->filial->getNmfilial(), 30);
            }else{
                if ($this->empresa->getIdtpijurempr() == 'F'){
                    $linha .= '1';
                }else{
                    $linha .= '2';
                }
                $nrinsjurempr = trim($this->empresa->getNrinsjurEmpr());
                if (strlen($nrinsjurempr) > 14){
                   $msg = "Código da inscrição da empresa deve ter no máximo 14 posições.";
               }
               $linha .= $this->empresa->getNrinsjurempr();
               $nmemprfili = str_pad($this->empresa->getNmempresa(), 30);
            }
            
            //Código do convênio no banco - Posição: 33 a 52 - Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Agência Cônvênio - Posição: 53 a 57 - Picture: 9(05)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            
            if($digitoAgencia){
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            }
            
            //Digito verificador da Agência Convênio - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da Conta Corrente Convênio - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Dígito Verificador da Conta Convênio - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            // Dígito Verificador Ag/Conta Convênio - Posição: 72 a 72 - Picture: 9(1)
            $linha.= ' ';

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Mensagem - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ',40);

            //Endereço - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_repeat(' ',30);

            //Nº Endereço - Posição: 173 a 177 - Picture: 9(5)
            $linha.= str_repeat('0',5);

            //Complemento Endereço - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //CIDADE - Posição: 193 a 212 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //CEP + COMPLEMENTO - Posição: 213 a 220 - Picture: 9(8)
            $linha.= str_repeat('0',8);

            //UF - Posição: 221 a 222 - Picture: X(2)
            $linha.= $this->estadoEF;
            
            //Indicativo de Forma de Pagamento do Compromisso - Posição: 223 a 224 - Picture: 9(2)
            $linha.= str_repeat(' ',2);
            
            //CNAB - Posição: 223 a 230 - Picture: X(6)
            $linha.= str_repeat(' ',6);
            
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";
            
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function formaPagamento($cdbanco, $boletos)
    {
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
        }
        
        switch ($idbaixautilp)
        {
            case 'B':
                if($cdbanco == self::CODIGO_COMPENSACAO_BTG)
                    $this->formapgto = '30';
                else
                    $this->formapgto = '31';
                break;
            case 'G': $this->formapgto = '17';
                break;
            case 'H': $this->formapgto = '18';
                break;
            case 'I': $this->formapgto = '11';
                break;
            case 'O': $this->formapgto = '10';
                break;
            case 'R': $this->formapgto = '11';
                break;
            case 'U': $this->formapgto = '11';
                break;
            case 'X': $this->formapgto = '22';
                break; 
            case 'J': $this->formapgto = '21';
                break;
            case 'V': $this->formapgto = '25';
                break;
            case 'Z': $this->formapgto = '45';
                break;
            case 'F': $this->formapgto = '45';
                break;
            case 'G': $this->formapgto = '17';
                break;
            default:
                if ($cdbanco == self::CODIGO_COMPENSACAO_BTG)
                     $this->formapgto = '01';
                else $this->formapgto = '03';
                break;
        }

        return $this->formapgto;
    }


    /**
     * retorna a data atual no formato ddmmYYYY
     * @return string
     */
    private function getDataGeracao(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('d').$dataatual->format('m').$dataatual->format('Y');
    }

    /**
     * @return string
     */
    private function getHorarioAtual(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('H').$dataatual->format('i').$dataatual->format('s');
    }


    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }


    /**
     * @param \Fiscal\Entities\Boletpagar $boleto
     * @param $cdbanco
     * @throws Exception
     * @internal param $formapgto
     * @internal param array $boletos
     * @return string
     */
    public function geraDetalheSegmentoA(Boletpagar $boleto, $cdbanco){
        try {

            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_A;

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            
            //Busca tipo de baixa
            /** @var $tipobaixapag Tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByCdtpbaixapagAndNrorg($boleto->getCdtpbaixapag(),
                $this->nrorg );

            //Código da Câmara Centralizadora - Posição: 18 a 20 - Picture: 9(03)
            if($this->formapgto == '01' or $this->formapgto == '03' or $this->formapgto == '45'){
                if($tipobaixapag->getIdbaixautilp() == self::TIPO_BAIXA_PIX)
                        $linha.= '009'; //PIX
                elseif($cdbanco != self::CODIGO_COMPENSACAO_BTG){
                        if(floatval($boleto->getVrfinalbpag()) > floatval($tipobaixapag->getVrmaxdoccnab()))
                            if($tipobaixapag->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO)
                                 $linha.= '018'; //TED CIP
                            else $linha.= '810'; //TED STR
                        else $linha.= '700'; //DOC
                    }else{
                        $linha.= '000'; //CC
                    }
                    
                if (!$boleto->getIdchavepix()){

                    //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                    $linha.= str_pad($cdbanco,3,'0',STR_PAD_LEFT);
                    
                    if($tipobaixapag->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                        
                        //Ag. Mantenedora da Cta do Favor. - Posição: 24 a 28 - Picture: 9(05)
                        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
                        $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
                        
                        if($digitoAgencia){
                            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
                        } else {
                            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
                        }
                        
                        //Dídito verificador da agência - Posição: 29 a 29 - Picture: X(01)
                        $linha.= ' ';
    
                        //Número da Conta Corrente - Posição: 30 a 41 - Picture: 9(12)
                        $linha.= self::preenche_brancos('',12);
    
                        //Dígito Verificador da Conta - Posição: 42 a 42 - Picture: X(01)
                        $linha.= '0';
    
                        //Dígito Verificador da AG/Conta - Posição: 43 a 43 - Picture: X(01)
                        $linha.= ' ';
                    }
                    else{
                        //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                        /** @var $bancoFornecedor Banco */
                        $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());
    
                        // Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras, entrará nesse if
                        if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                            //Se na máscara da agência NÃO tem '-' 
                            if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
                                //Agencia
                                $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5)
                                    ,5,'0',STR_PAD_LEFT);
                                //DV
                                $linha.= ' ';
                            }
                            else{
                                // Caso a máscara da agência TENHA '-' 
                                //Agencia
                                $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,
                                        strpos($bancoFornecedor->getCdpictagen(),'-'))
                                    ,5,'0',STR_PAD_LEFT);
    
                                //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                                //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                                if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                        ,strpos($bancoFornecedor->getCdpictagen(),'-')))>0){
    
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                        ,$pos,1);
                                }
                                else{
                                    $linha.= ' ';
                                }
                            }
                        }
                        else{
                            // Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com as máscaras, entrará nesse else 
                            if(strpos($boleto->getCdagenbolp(),'-')=== false){
                                // se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' 
                                //Agencia
                                $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);
    
                                //DV
                                $linha.= ' ';
                            }
                            else{
                                // caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' 
                                //Agencia
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,
                                    strpos($boleto->getCdagenbolp(),'-')
                                )),5,'0',STR_PAD_LEFT);
    
                                //DV
                                $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,$pos,1);
                            }
                        }
    
                        //Número da conta corrente favorecido - Posição: 030 a 041 - Picture: 9(11)
                        //MAIS
                        //Dígito Verificador da Conta - Posição: 42 a 42 - Picture: X(01)
    
                        // Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras, entrará nesse if
                        if($bancoFornecedor and $bancoFornecedor->getCdpictcont()){
                            //{ se na máscara da conta corrente TEM '-' }
                            if(strpos($bancoFornecedor->getCdpictcont(),'-')){
    
                                //tamDifVer só pode assumir os valor 1 ou 2.
                                // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                                // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 
    
                                $tamDigVer = strlen(trim(substr($bancoFornecedor->getCdpictcont(),
                                    strpos($bancoFornecedor->getCdpictcont(),'-')+1,
                                    2)));
                                if($tamDigVer>1){
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                            strpos($bancoFornecedor->getCdpictcont(),'-'))),
                                        12,'0',STR_PAD_LEFT);
    
                                    //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                                    //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                    //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                                    
                                    if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                            ,strpos($bancoFornecedor->getCdpictcont(),'-')))>0){
                                        //DV
                                        $pos = strpos($bancoFornecedor->getCdpictcont(),'-')+1 ;
                                        $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                            ,$pos,1);
                                    }
                                    else{
                                        $linha.= ' ';
                                    }
    
                                }
                                else{
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        -1
                                    )),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp())),
                                        -1);
    
                                }
    
                            }
                            else{
                                
                                // CASO a máscara da conta corrente NÃO tenha '-' 
                                
                                if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_BTG){
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                    ),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= substr($boleto->getCdccorbolp(),-1);
    
                                }
                                else{
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                    ),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= ' ';
                                }
                            }
    
                        }
                        // Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com as máscaras, entrará nesse else 
                        else{
                            if(strpos($boleto->getCdccorbolp(),'-')){
                                //se o que foi digitado no campo de conta Corrente do favorecido da tela de boleto tem '-'
                                //CC
    
                                //tamDifVer só pode assumir os valor 1 ou 2.
                                // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                                // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 
    
                                $tamDigVer = strlen(trim(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1,
                                    2)));
    
                                if($tamDigVer>1){
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                            strpos($boleto->getCdccorbolp(),'-'))
                                    ),11,'0',STR_PAD_LEFT);
                                    $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                        strpos($boleto->getCdccorbolp(),'-')+1,1));
                                    //DV
                                    $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                        strpos($boleto->getCdccorbolp(),'-')+2,1));
                                }
                                else{
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                            strpos($boleto->getCdccorbolp(),'-'))
                                    ),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                        strpos($boleto->getCdccorbolp(),'-')+1));
                                }
                            }
                            //CASO o que foi digitado no campo de conta corrente do favorecido da tela de Boleto NÃO tenha '-'
                            else{
                                if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_BTG){
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                    ),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= substr($boleto->getCdccorbolp(),-1);
                                }
                                else{
                                    //CC
                                    $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                    ),12,'0',STR_PAD_LEFT);
    
                                    //DV
                                    $linha.= ' ';
                                }
                            }
                        }
                        //Dígito Verificador da AG/Conta - Posição: 43 a 43 - Picture: X(01)
                        $linha.= ' ';
                    }
                }else{
                    $linha.= str_repeat('0', 23);
                }
                
                //Nome do favorecido - Posição: 44 a 73 - Picture: X(30)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

                /** @var $contaPagar Contapagar */
                //Número do documento atribuído pela empresa - Posição: 74 a 93 - Picture: X(20)
                $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                    $boleto->getCdfilial(),
                    $boleto->getNrorg());
                $nrnotafisc = $contaPagar->getLanctonf()->getNrnotafisc();
                if($nrnotafisc){
                    $linha.= str_pad($nrnotafisc,20,' ',STR_PAD_RIGHT);
                } else{
                    $linha.= self::preenche_brancos(' ',20);
                }
                
                //Data do Pagamento - Posição: 94 a 101 - Picture: 9(08)
                $linha.= $boleto->getDtvenbolpag()->format('dmY');

                //Tipo da Moeda - Posição: 102 a 104 - Picture: X(03)
                $linha.= self::MOEDA;

                //Quantidade da moeda - Posição: 105 a 119 - Picture: 9(15)
                $linha.= str_pad('',15,'0', STR_PAD_RIGHT);

                //Valor do Pagamento - Posição: 120 a 134 - Picture: 9(13)V9(2)
                $linha.= str_pad(str_replace(array('.',','),'',
                    number_format($boleto->getVrfinalbpag(),2)
                ),15,'0',STR_PAD_LEFT);
                
                //Número do documento atribuido pelo banco - Posição: 135 a 154 - Picture: X(20)
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
                } else{
                    $linha.= self::preenche_brancos(' ',20);
                }
                
                //Data Real da Efetivação Pagto - Posição: 155 a 162 - Picture: 9(08)
                $linha.= str_pad('',8,'0');

                //Valor Real da Efetivação do Pagto - Posição: 163 a 177 - Picture: 9(15)
                $linha.= str_pad('',15,'0');

                //Outras Informações - Posição: 178 a 217 - Picture: X(40)
                $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),40,' ',STR_PAD_RIGHT);

                //Compl. Tipo Serviço - Posição: 218 a 219 - Picture: 9(02)
                if($this->formapgto == '03')
                    $linha.= '03';
                else
                    $linha.= '13';

                //Codigo finalidade da TED - Posição: 220 a 224 - Picture: X(5)
                $linha.= self::preenche_brancos(' ',5);
              
                //Complemento de finalidade pagto. - Posição: 225 a 226 - Picture: X(2)
                $linha.= self::preenche_brancos(' ',2);
                
                //Uso Exclusivo FEBRABAN/CNAB. - Posição: 227 a 229 - Picture: X(3)
                $linha.= self::preenche_brancos(' ',3);
                
                //Aviso ao Favorecido - Posição: 230 a 230 - Picture: 9(1)
                $linha.= str_pad('',1,'0');
                
                //Códigos das Ocorrências p/ Retorno - Posição: 231 a 240 - Picture: X(10)
                $linha.= self::preenche_brancos(' ',10);
              
            }
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraDetalheSegmentoB(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMETO_B;
            
            if (!$boleto->getIdchavepix()){
                //CNAB - Posição: 15 a 17 - Picture: 9(03)
                $linha.= str_repeat(' ',3);
                
                //Tipo de Inscrição do Favorecido - Posição: 18 a 18 - Picture: 9(01)
                //Nº de Inscrição do Favorecido - Posição: 19 a 32 - Picture: 9(14)
                if($boleto->getNrinscnabbop()){
                    if(strlen(Util::campoCnabValido($boleto->getNrinscnabbop())) == 11){
                        $linha.= self::TIPO_INS_JUR_CPF;
                    }else{
                        $linha.= self::TIPO_INS_JUR_CNPJ;
                    }
                    $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
                }else{
                    if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                        $linha.= self::TIPO_INS_JUR_CPF;
                    }else{
                        $linha.= self::TIPO_INS_JUR_CNPJ;
                    }
                    $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
                }
    
    
                //Nome do logradouro do favorecido - Posição: 33 a 67 - Picture: X(35)
                $endforn = $boleto->getForneced()->getEndefornecobr();
                if(! $endforn)
                    $endforn = $boleto->getForneced()->getEndeforneprinc();
                $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),35,' ',STR_PAD_RIGHT);
    
                //Número do imóvel do favorecido - Posição: 63 a 72 - Picture: 9(05)
                $linha.= str_repeat('0',5);
    
                //Complemento do imóvel - Posição: 73 a 87 - Picture: X(15)
                $linha.= str_repeat(' ',15);
    
                //Bairro do favorecido - Posição: 88 a 102 - Picture: 9(15)
                $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);
                 
                //Município do favorecido - Posição: 103 a 117 - Picture: X(15)
                $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
                $linha.= str_pad($municipio->getNmmunicipio(), 15);
    
                //CEP - Posição: 118 a 122 - Picture: 9(05)
                if($endforn->getNrcepforn()){
                    $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
                }else{
                    $linha.= str_repeat('0',5);
                }
    
                //Complemento do CEP - Posição: 123 a 125 - Picture: X(03)
                if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
                    $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);
    
                }else{
                    $linha.= str_repeat(' ',3);
                }

                //Estado - Posição: 126 a 127 - Picture: X(02)
                $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);
    
                //Data do vencimento (Nominal) - Posição: 128 a 135 - Picture: 9(08)
                $linha.= $boleto->getDtvenbolpag()->format('dmY');
    
                //Valor do documento (Nominal) - Posição: 136 a 150 - Picture: 9(13)V9(2)
                $linha.= str_pad(str_replace(array('.',','),'',
                    number_format($boleto->getVrfinalbpag(),2)
                ),15,'0',STR_PAD_LEFT);
    
                //Valor do abatimento - Posição: 151 a 165 - Picture: 9(13)V9(2)
                $linha .= str_repeat('0', 15);
    
                //Valor do desconto - Posição: 166 a 180 - Picture: 9(13)V9(2)
                $linha .= str_repeat('0', 15);
    
                //Valor da mora - Posição: 181 a 195 - Picture: 9(13)V9(2)
                $linha .= str_repeat('0', 15);
    
                //Valor da multa - Posição: 196 a 210 - Picture: 9(13)V9(2)
                $linha .= str_repeat('0', 15);
    
                // Código/Documento do Favorecido - Posição: 211 a 225 - Picture: X(15)
                $linha .= str_repeat(' ', 15);
                
                //Aviso ao Favorecido - Posição: 226 a 226 - Picture: 9(1)
                $linha .= str_repeat('0', 1);
                
                // Uso Exclusivo para o SIAPE- Posição:  227 a 232 - Picture: 9(6) 
                $linha .= str_repeat('0', 6);
                
                // Código ISPB - Posição:  233 a 240 - Picture: 9(8) 
                $linha .= str_repeat('0', 8);

            }else{
                //TIPO CHAVE
                //Posição: 15 a 16
                //Picture: X(2)
                $linha.= $boleto->getIdchavepix();
                
                //Complemento do registro
                // Posicao 17 a 17
                //Picture X(1)
                $linha.= str_repeat(' ', 1);
                
                //Tipo de incrição do fornecedor - Posição: 18 a 18 - Picture: 9(01)
                //Código do CPF/CNPJ - Posição: 19 a 32 - Picture: 9(14)
                if($boleto->getNrinscnabbop()){
                    if(strlen(Util::campoCnabValido($boleto->getNrinscnabbop())) == 11){
                        $linha.= self::TIPO_INS_JUR_CPF;
                    }else{
                        $linha.= self::TIPO_INS_JUR_CNPJ;
                    }
                    $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
                }else{
                    if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                        $linha.= self::TIPO_INS_JUR_CPF;
                    }else{
                        $linha.= self::TIPO_INS_JUR_CNPJ;
                    }
                    $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
                }
                
                // TXID - Opcional
                // Posicao 33 a 67
                // Picture X(35)
                $linha.= str_repeat(' ', 35);
                
                // INFORMAÇÕS ENTRE USUÁRIOS  - Opcional
                // Posicao 68 a 127
                // Picture X(60)
                $linha.= str_repeat(' ', 60);
                
                // CHAVE PIX  - obrigatorio
                // Posicao 128 a 226
                // Picture X(100)
                $linha.= str_pad($boleto->getChavepix(), 99);
                
                // Uso Exclusivo para o SIAPE
                // Posicao 227 a 232
                //Picture 9(6)
                $linha.= str_repeat('0', 6);
                
                // Código ISPB
                // Posicao 233 a 240
                // Picture 9(8)
                $linha.= str_repeat('0', 8);
                
            }
            
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_J;

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }


            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }


            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome do Beneficiário - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Título - Posição: 100 a 114 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Desconto + Abatimento - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Quantidade de moeda - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Nº do Docto Atribuído pela Empresa - Posição: 183 a 202 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Nº do Docto Atribuído pelo Banco - Posição: 203 a 222 - Picture: X(20)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
            } else{
                $linha.= str_repeat(' ',20);
            }

            //Código da Moeda - Posição: 223 a 224 - Picture: X(02)
            $linha.= str_repeat('0',2);

            //CNAB - Posição: 225 a 230 - Picture: X(6)
            $linha.= str_repeat(' ',6);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ52(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_J;
            
            //CNAB - Posição: 15 a 15 - Picture: X(01)
            $linha.= str_repeat(' ',1);

            //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Identificação Registro - Posição: 18 a 19 - Picture: 9(02)
            $linha.= self::SEGMENTO_J52;
            
            /*
             * #### PAGADOR ####
             */
            //Tipo de Inscrição Pagador - Posição: 20 a 20 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CPF/CNPJ do Pagador - Posição: 21 a 35 - Picture: 9(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Nome do Pagador - Posição: 36 a 75 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
            
            /*
             * #### Beneficiário ####
             */
            //Tipo de Inscrição Beneficiário - Posição: 76 a 76 - Picture: 9(01)
            if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                $linha.= self::TIPO_INS_JUR_CPF;
            }else{
                $linha.= self::TIPO_INS_JUR_CNPJ;
            }
            
            //Número de Inscrição do Beneficiário - Posição: 77 a 91 - Picture: 9(01)
            $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);

            //Nome do Beneficiário - Posição: 92 a 131 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
            
            /*
             * #### SACADOR ####
             */
            //Tipo de Inscrição Sacador - Posição: 132 a 132 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //Número de Inscrição do Sacador - Posição: 133 a 147 - Picture: 9(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Nome do Sacador - Posição: 148 a 187 - Picture: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
            
            //CNAB - Posição: 188 a 240 - Picture: X(53)
            $linha.= str_repeat(' ',53);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoO(Boletpagar $boleto){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_O;

            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            
            $tiposbaixapag = array(
                Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, 
                Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, 
                Tipobaixapag::TIPO_BAIXA_FGTS,
                self::TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL
            );
            
            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if(in_array($boleto->getTpbaixapag()->getIdbaixautilp(), $tiposbaixapag)){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome Concessionária - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data de Vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Data do Pagamento - Posição: 100 a 107 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Pagamento - Posição: 108 a 122 - Picture: 9(13)V(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

            //Nº do Docto Atribuído pela Empresa - Posição: 123 a 142 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Número Documento atribuido pelo Banco - Posição: 143 a 162 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //Posição: CNAB 163 a 230 - Picture: X(68)
            $linha.= str_repeat(' ',68);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function geraSegmentoN($boleto)
    {
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº Seqüencial do Registro no Lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código de Segmento no Reg. Detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMENTO_N;
            
            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            
            //Número Documento do Cliente - Posição: 18 a 37 - Picture: 9(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Número Documento atribuido pelo Banco - Posição: 38 a 57 - Picture: X(20)
            $linha.= str_repeat(' ',20);
            
            //Nome do Contribuinte - Posição: 58 a 87 - Picture: 9(30)
            $linha.= str_pad(Util::campoCnabValido(substr($this->nomerazempresa, 0, 30)), 30, ' ');
            
            //Data de Pagamento - Posição: 88 a 95 - Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Valor do Total do Pagamento - Posição: 96 a 110 - Picture: 9(13)V2(02)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            //Identificação do tributo - Posição: 111 a 230
            if ($this->formapgto == '16')
                $linha = self::darfNormal($linha, $boleto);
            elseif($this->formapgto == '17')
                $linha = self::GPS($linha, $boleto);
            elseif ($this->formapgto == '18')
                $linha =self::darfSimples($linha, $boleto);
            elseif($this->formapgto == '22')
                $linha =self::GARE($linha, $boleto);
            elseif($this->formapgto == '21')
                $linha =self::DARJ($linha, $boleto);
            
            //Ocorrencia  - Posição: 231 a 240 - Picture: X(30)
            $linha.= str_repeat(' ', 10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function darfNormal($linha, $boleto)
    {
        // Código da Receita do Tributo  - Posição: 111 a 116 - Picture: 9(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia)
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 6);

        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '01'; //CPF
        else
            $linha.= '02'; //CNPJ

        //CPF ou CNPJ do contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);

        // Código de Identificação do Tributo - Posição: 133 a 134 - Picture: X(002)
        $linha.= '16'; //DARF  Normal

        //Período de Apuração - Posição: 135 a 142 - Picture: 9(08)
        if ($boleto->getDtapuracguia())
             $linha.= $boleto->getDtapuracguia()->format('dmY');
        else $linha.= atr_repeat('0', 8);
        
        //Número de referência - Posição: 143 a 159 - Picture: 9(17)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia)
             $linha.= str_pad($nrrefeguia,17,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 17);

        //Valor Principal - Posição: 160 a 174 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

        //Valor da Multa - Posição: 175 a 189 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 190 a 204 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);

        //Data do Vencimento - Posição: 205 a 2012 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //CNAB - Posição: 213 a 230 - Picture: X(18)
        $linha.= str_repeat(' ', 18);
        
        return $linha;
    }

    public function darfSimples($linha, $boleto)
    {
        // Código da Receita do Tributo  - Posição: 111 a 116 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia)
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 6);

        //Tipo de Identificação do Contribuinte - Posição: 117 a 118 - Picture: 9(1)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '01'; //CPF
        else
            $linha.= '02'; //CNPJ

        //Identificação do Contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);

        // Código de Identificação do Tributo - Posição: 133 a 134 - Picture: X(002)
        $linha.= '16'; //DARF  Simples

        //Período de Apuração - Posição: 135 a 142 - Picture: 9(08)
        if ($boleto->getDtapuracguia())
             $linha.= $boleto->getDtapuracguia()->format('dmY');
        else $linha.= str_repeat('0', 8);
        
        //Valor da Receita Bruta Acumulada - Posição:  143 a 157 - Picture:  9(13)V(2)
        $linha.= str_pad(Util::campoCnabValido($boleto->getVroutrentguia()),15, STR_PAD_LEFT);
        
        //Percentual - Posição: 158 a 164 - Picture: 9(5)V2(2)
        $linha.= str_pad(Util::campoCnabValido($boleto->getVrpercenttrib()),7, STR_PAD_LEFT);
        
        //Valor Principal - Posição: 165 a 179 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

        //Valor da Multa - Posição: 180 a 194 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 195 a 209 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);

        //CNAB - Posição: 210 a 230 - Picture: X(21)
        $linha.= str_repeat(' ', 21);

        return $linha;
    }

    public function GPS($linha, $boleto)
    {
        // Código da Receita do Tributo  - Posição: 111 a 116 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia)
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 6);

        //Tipo de Identificação do Contribuinte - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '1 '; //CPF
        else
            $linha.= '2 '; //CNPJ

        //Identificação do Contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);

        // Código de Identificação do Tributo - Posição: 133 a 134 - Picture: X(002)
        $linha.= '17'; //GPS

        //Mês e ano da Competência - Posição: 135 a 140 - Picture: 9(06)
        if ($boleto->getDtapuracguia())
            $linha.= $boleto->getDtapuracguia()->format('mY');
        else
            $linha.= str_repeat('0', 6);
        
        //Valor previsto do pagamento do INSS - Posição: 141 a 155 - Picture: 9(13)V2(02)
        $vrprevisto = $boleto->getVrfinalbpag() - ($boleto->getVroutrentguia() + $boleto->getVrmultabpag() + $boleto->getVrmordiabpag());
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vrprevisto,2)),15,'0',STR_PAD_LEFT);

        //Valor de Outras Entidades - Posição: 156 a 170 - Picture: 9(013)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);

        //Atualização Monetária - Posição: 171 a 185 - Picture: 9(013)V2(02)
        $vratualizacao = $boleto->getVrmultabpag() + $boleto->getVrmordiabpag();
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vratualizacao,2)),15,'0',STR_PAD_LEFT);

        //CNAB - Posição: 186 a 230 - Picture: X(45)
        $linha.= str_repeat(' ', 45);
        
        return $linha;
    }
    
    public function GARE($linha, $boleto){
        
        // Código da Receita do Tributo  - Posição: 111 a 116 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia)
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 6);

        //Tipo de Identificação do Contribuinte - Posição: 117 a 118 - Picture: 9(1)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '1 '; //CPF
        else
            $linha.= '2 '; //CNPJ

        //Identificação do Contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);

        // Código de Identificação do Tributo - Posição: 133 a 134 - Picture: X(002)
        $linha.= '05'; 

        //Data de Vencimento - Posição: 135 a 142 - Picture: 9(8)
        if ($boleto->getDtapuracguia())
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        else
            $linha.= str_repeat('0', 8);
        
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        //Inscrição Estadual / Código do Município / Número Declaração - Posição: 143 a 154 - Picture: 9(12)
        $linha.= str_pad($empresa->getCdinscesta(),12, '0', STR_PAD_LEFT);
        
        //Dívida Ativa / N. Etiqueta - Posição: 155 a 167 - Picture: 9(13)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!Empty($nrrefeguia))
            $linha.= str_pad(trim(is_numeric($nrrefeguia)), 13, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 13);
            
        //Período de Referência - Posição: 168 a 173 - Picture: 9(6)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= self::formataData($boleto->getDtapuracguia());
        else
            $linha.= str_repeat('0', 6);
            
        //Número da Parcela / Notificação - Posição: 174 a 186 - Picture: 9(13)
        $linha.= str_pad($boleto->getNrparcbolpag(),13, '0', STR_PAD_LEFT);
        
        //Valor da Receita - Posição: 187 a 201 - Picture: 9(15)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros / Encargos - Posição: 202 a 215 - Picture: 9(14)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 216 a 229 - Picture: 9(14)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //CNAB - Posição: 230 a 230 - Picture: X(1)
        $linha.= str_repeat(' ', 1);
        
        return $linha;
    }
    
    public function DARJ($linha, $boleto){
        
        // Código da Receita do Tributo  - Posição: 111 a 116 - Picture: X(6)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia)
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        else $linha.= str_repeat('0', 6);

        //Tipo de Identificação do Contribuinte - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '1 '; //CPF
        else
            $linha.= '2 '; //CNPJ

        //Identificação do Contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //Inscrição Estadual / Código do Município / Número Declaração - Posição: 133 a 140 - Picture: 9(8)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        $linha.= str_pad($empresa->getCdinscesta(),8, '0', STR_PAD_LEFT);
        
        //Número do Documento Origem -  Posição: 141 a 156 - Picture: 9(16)
        $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
        $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),16,'0',STR_PAD_LEFT);
        
        //Valor Principal - Posição: 157 a 171 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag() - $boleto->getVrmordiabpag() + $boleto->getVrmultabpag(),2)),15,STR_PAD_LEFT);
        
        //Valor Atualização Monetária - Posição: 172 a 186 - Picture: 9(13)V2(02)
        $linha.= str_repeat('0', 15);
        
        //Valor dos Juros/ Encargos - Posição: 187 a 201 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 202 a 216 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 217 a 224 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Período de Referência - Posição: 225 a 230 - Picture: 9(06)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= $boleto->getDtvenbolpag()->format('mY');
        else
            $linha.= str_repeat('0', 6);
        
        return $linha;
    }
    
    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraDetalheSegmentoI(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= self::SEGMETO_I;

            //CNAB - Posição: 15 a 15 - Picture: 9(01)
            $linha.= str_repeat(' ',1);
            
            //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
            $linha.= '01';
            
            //Número do contrato  - Posição: 18 a 27 - Picture: 9(10)
            $linha.= str_repeat('0',10);
            
            //Número do documento - Nota Fiscal - Posição: 28 a 42 - Picture: X(15)
            $linha.= str_repeat(' ',15);
            
            //Data da Compra - Posição: 43 a 50 - Picture: 9(08)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $notafiscal = self::getNotaFiscal($contaPagar->getNrlanctonf(), $contaPagar->getCdfilial(), $contaPagar->getNrorg());
            $linha.= $notafiscal->getDtemissao()->format('dmY');
            
            //Regime de Encargos Financeiros - Posição: 51 a 51 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Modalidade de Encargo - Posição: 52 a 53 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Taxa de operação a Desconto - Posição: 54 a 63 - Picture: 9(03)v7
            $linha.= str_repeat('0',10); //verificar
            
            //Data do Primeiro vencimento - Posição: 64 a 71 - Picture: 9(08)
            $linha.= str_repeat('0',8);
            
            //Data do Vencimento Final - Posição: 72 a 79 - Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Tipo de Vencimento da Parcela - Posição: 80 a 80 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Periodicidade Prazo de vencimento - Posição: 81 a 82 - Picture: 9(02)
            $linha.= str_repeat('0',2);
            
            //Quantidade de Parcelas - Posição: 83 a 84 - Picture: 9(02)
            $linha.= $boleto->getNrparcbolpag();
            
            //Nosso Número - Posição: 85 a 104 - Picture: X(20)
            $linha.= str_repeat(' ',20);
            
            //Forma de Pagamento - Posição: 105 a 105 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Valor de encargos da operação - Posição: 106 a 120 - Picture: 9(13)v2
            $linha.= str_repeat('0',15);
            
            //Pagamento do IOF - Posição: 121 a 121 - Picture: 9(01)
            $linha.= str_repeat('0',1);
            
            //Valor do IOF - Posição: 122 a 136 - Picture: 9(15)v2
            $linha.= str_repeat('0',15);
            
            //Valor da Operação - Posição: 137 a 151 - Picture: 9(13)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            //Juros de Mora - Posição: 152 a 166 - Picture: 9(13)v2
            $linha.= str_repeat('0',15);
            
            //Valor IOF sobre atraso - Posição: 167 a 181 - Picture: 9(13)v2
            $linha.= str_repeat(' ',15);
            
            //Multa - Posição: 182 a 196 - Picture: 9(13)v2
            $linha.= str_repeat(' ',15);
            
            //CNAB - Posição: 197 a 240 - Picture: X(44)
            $linha.= str_repeat(' ',44);
            

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }
    

    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
    
    /**
     * * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrorg
     * @return Notafiscal
     */
    private function getNotaFiscal($nrlanctonf, $cdfilial, $nrorg){
        /** @var $notafiscal Notafiscal */
        $notafiscal = Notafiscal::findOneByNrlanctonfAndCdfilialAndNrorg($nrlanctonf, $cdfilial, $nrorg);
        return $notafiscal;
    }

    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote($somavalores = null){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            $linha .= str_pad($this->num_header_lote, 4, '0', STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::ID_TP_ARQUIVO_TRAILER_LOTE;

            //CNAB - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de Registro do lote - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

            //Somatorio dos valores - Posição: 24 a 41 - Picture: 9(16)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($this->vrPagtLote,2)
            ),18,'0',STR_PAD_LEFT);

            //Somatório da quantidade de moeda - Posição: 42 a 59 - Picture: 9(18)
            $linha .= str_repeat('0', 18);

            //Número Aviso de Débito - Posição: 60 a 65 - Picture: 9(6)
            $linha .= str_repeat('0', 6);

            //CNAB - Posição: 66 a 230 - Picture: X(165)
            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 175);

            //Zera o contador de registros do lote
            $this->qtdRegiLote = 0;

            //Zera o sequencial de registros do lote
            $this->seqRegLote = 0;

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){

        try { 
            //Quantidade de registro do arquivo
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_BTG;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::LOTE_SERVICO_TRAILER_ARQUIVO . self::ID_TP_ARQUIVO_TRAILER;

            //CNAB - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtLote, 6, '0', STR_PAD_LEFT);

            //Quantidade de registros do arquivo - Posição: 24 a 29 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiArq, 6, '0', STR_PAD_LEFT);
            
            //Qtde. de Contas Concil. - Posição: 30 a 35 - Picture: 9(6)
            $linha .= str_repeat('0', 6);
            
            //CNAB - Posição: 36 a 240 - Picture: X(205)
            $linha .= str_repeat(' ', 205);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}