<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Contareceber;
use Financeiro\Entities\Emprbanc;
use Financeiro\Util\Util;
use Geral\Entities\Banco;
use Geral\Entities\Cliente;
use Geral\Entities\Empresa;
use Geral\Entities\Endeclie;
use Geral\Entities\Municipio;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Geral\Entities\Contcorr;
use Geral\Entities\Filial;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;

/**
 * Class GeraCnabSantanderService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabBanrisulService extends GeraCnabService{

   const LOTE_SERVICO = '0000';
   const REGISTRO = '0';
   const TIPO_INSCRICAO_CEDENTE_CNPJ = '2';
   const TIPO_INSCRICAO_CEDENTE_CPF = '1';
   const CODIGO_REMESSA = '1';
   const LAYOUT = '103';
   const LOTE_SERVICO_HEADER_LOTE = '0001';
   const TIPO_REGISTRO_HEADER_LOTE = '1';
   const TIPO_OPERACAO = 'R';
   const TIPO_SERVICO_HEADER_LOTE = '01';
   const VERSAO_ARQUIVO_LOTE = '060';
   const TIPO_REGISTRO_SEGMENTO = '3';
   const SEGMENTO_P = 'P';
   const SEGMENTO_Q = 'Q';
   const SEGMENTO_R = 'R';
   const SEGMENTO_Y = 'Y';
   const ID_OPERACAO_CNAB_INCLUSAO = '01';
   const ID_OPERACAO_CNAB_EXCLUSAO = '02';
   const ID_OPERACAO_CNAB_ALTERACAO = '06';
   const TIPO_DOCUMENTO = '1';
   const ESPECIE_TITULO = '99';
   const IDTITULO_ACEITO = 'N';
   const CODIGO_BAIXA = '0';
   const CODIGO_MOEDA = '9';
   const PGTOPARCIAL = '1';
   const CODIGO_MULTA_1 = '1';
   const CODIGO_MULTA_2 = '2';
   const CODIGO_DESCONTO_2 = '0';
   const IDENTIFICACAO_ARQUIVO_TRAILLER_LOTE = '5';
   const IDENTIFICACAO_ARQUIVO_TRAILLER = '9';
   const LOTE_SERVICO_2 = '0001';
   const SEGMENTO_S = 'S';
   const FONTE = '01';
   const LOTE_DO_SERVICO = '9999';
   
    protected $entityManager;
    protected $titulosAReceberFactory;
    protected $inQtCobrSimples;
    protected $vrCobrSimples;
    protected $inQtCobrVinc;
    protected $vrCobrVinc;
    protected $inQtCobrCalc;
    protected $vrCobrCalc;
    protected $inQtCobrDesc;
    protected $vrCobrDesc;

    /**
     * @param EntityManager $entityManager
     * @param TitulosAReceberFactory $titulosAReceberFactory
     */
    public function __construct(EntityManager $entityManager, TitulosAReceberFactory $titulosAReceberFactory)
    {
        $this->entityManager          = $entityManager;
        $this->titulosAReceberFactory = $titulosAReceberFactory;
    }

    /**
     * Gera Header Arquivo
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnab
     * @param $cdreducont
     * @param null $carteira
     * @return mixed|void
     */
    public function geraHeaderArquivo(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL
    ){
        
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "w+");
        
        // CAMPO: Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = Util::campoCnabValido($banco->getCdbanccnab());
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= Util::campoCnabValido(self::REGISTRO);
        
        // CAMPO: CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: 9(9)
        //Default : Brancos
        $arquivo .= self::preenche_brancos('', 9);
        
        //DADOS EMPRESA
        
        // CAMPO: Tipo Inscrição
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
        if($cdfilial){
            $filial = Filial::findOneByCdfilial($cdfilial);
            if($filial->getIdtpijurfili() == 'J'){
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else {
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }else {
            if (strlen($idtpijurempr) == '14'){
              $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else{
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }
        
        $arquivo .= Util::campoCnabValido($stTpInscCedente);
        
        // CAMPO: Número Inscrição
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(14)
        
        // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o NRINSJUR da filial e não da empresa.
        if($cdfilial) {
            $filial = Filial::findOneByCdfilial($cdfilial);
            if ($filial) {
                $nrinsjurempr = Util::campoCnabValido($filial->getNrinsjurfili());
            } else {
                $nrinsjurempr = Util::campoCnabValido($nrinsjurempr->getNrinsjurempr());
            }
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr, 14, "0", STR_PAD_LEFT));
        } else{
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 14, "0", STR_PAD_LEFT));
        }
        
        // CAMPO: Convênio 
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        //Banrisul: o código do beneficiário possui 13 dígitos 13 - Cd Convênio + 7 Zeros
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr->getCdcedenteban()){
            $arquivo .= str_pad($contcorr->getCdcedenteban(), 13, '0', STR_PAD_LEFT);
            $arquivo .= str_repeat('0',7);
        }else{
            $arquivo .= str_repeat('0',20);
        }
        
        // CAMPO: Agência 
        // POSIÇÃO: 053 a 057
        // PICTURE: 9(5)
        //Default : Brancos
        
        // CAMPO: DV 
        // POSIÇÃO: 058 a 058
        // PICTURE: 9(1)
        //Default : Brancos
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia()){
            $banco = Banco::findOneByCdbancoAndNrorg($cdbanco, $nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            if($digitoAgencia){
              $arquivo.= str_pad(substr($contcorr->getCdagencia(),0,5),6,'0',STR_PAD_LEFT);
            } else {
              $arquivo.= str_pad(str_pad(substr($contcorr->getCdagencia(),0,5),5,'0',STR_PAD_LEFT),6,'0',STR_PAD_RIGHT);
            }
        }else{
          $arquivo .= self::preenche_brancos('', 6);
        }
        
        // CAMPO: Conta 
        // POSIÇÃO: 059 a 070
        // PICTURE: 9(12)
        //Default : Brancos
        
        // CAMPO: DV 
        // POSIÇÃO: 071 a 071
        // PICTURE: 9(1)
        //Default : Brancos
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
            
        } else {
            $arquivo .= str_repeat('0',12);
        }
        
        // CAMPO: DV AG/CC
        // POSIÇÃO: 072 a 072
        // PICTURE: 9(1)
        //Default : Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Nome (Nome do Beneficiário (antigo Cedente))
        // POSIÇÃO: 073 a 102
        // PICTURE: 9(30)
        $nomeempresa = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        
        // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o nome da filial e não da empresa.
        if($cdfilial) {
            $filial = Filial::findOneByCdfilial($cdfilial);
            if($filial) {
                $nmempresa = substr($filial->getNmrazsocfili(), 0, 30);
            } else {
                $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
            }
            $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
        } else {
            $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
            $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
        }
        
        // CAMPO: Nome do Banco - BANRISUL
        // POSIÇÃO: 133 a 142
        // PICTURE: 9(30)
        $nmbanco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo .= Util::campoCnabValido(substr($nmbanco->getNmbanco(), 0, 30));
        
        // CAMPO: CNAB 
        // POSIÇÃO: 133 a 142
        // PICTURE: X(10)
        //Default : Brancos
        $arquivo .= self::preenche_brancos('', 10);
        
        //DADOS DO ARQUIVO
        
        // CAMPO: Código Remessa 
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        $arquivo .= Util::campoCnabValido(self::CODIGO_REMESSA); 
        
        // CAMPO: Data de Geração 
        // POSIÇÃO: 144 a 151
        // PICTURE: 9(8)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Hora de Geração
        // POSIÇÃO: 152 a 157
        // PICTURE: 9(6)
        $now = DateUtil::getDataAtual();
        $horaGeracao = $now->format('His');
        $arquivo .= $horaGeracao;
        
        // CAMPO: Sequência (NSA) 
        // POSIÇÃO: 158 a 163
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(substr($nrseqremessarec, -6));
        
        // CAMPO: Leiaute Arquivo
        // POSIÇÃO: 164 a 166
        // PICTURE: 9(3)
        //Default : 103
        $arquivo .= Util::campoCnabValido(self::LAYOUT); 
        
        // CAMPO: Densidade
        // POSIÇÃO: 167 a 171
        // PICTURE: 9(5)
        $arquivo .= str_repeat('0',5);
        
        // CAMPO: Reservado Banco
        // POSIÇÃO: 172 a 191
        // PICTURE: 9(20)
        //Default: Brancos
        
        // CAMPO: Reservado Empresa
        // POSIÇÃO: 192 a 211
        // PICTURE: 9(20)
        //Default: Brancos
        
        // CAMPO: CNAB 
        // POSIÇÃO: 212 a 240
        // PICTURE: 9(29)
        //Default: Brancos
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Header Lote
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $cdreducont
     * @param null $carteira
     * @return mixed|void
     */
    public function geraHeaderLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Banco 
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defaul: 041
        /**@var $banco Banco*/
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();

        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Defaul: 0001 a 9999
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);

        // CAMPO: Registro 
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
         //Defaul: 1
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_HEADER_LOTE);

        // CAMPO: Operação
        // POSIÇÃO: 009 a 009
        // PICTURE: X(1)
        //Defaul: 'R' = Arquivo Remessa
        $arquivo .= Util::campoCnabValido(self::TIPO_OPERACAO);

        // CAMPO: Serviço 
        // POSIÇÃO: 010 a 011
        // PICTURE: 9(2)
        $arquivo .= Util::campoCnabValido(self::TIPO_SERVICO_HEADER_LOTE);

        // CAMPO: CNAB 
        // POSIÇÃO: 012 a 013
        // PICTURE: X(2)
        //Defaul: Brancos
        $arquivo .= self::preenche_brancos('', 2);

        // CAMPO: Leiaute do Lote
        // POSIÇÃO: 014 a 016
        // PICTURE: X(3)
        //Defaul: 060
        $arquivo .= Util::campoCnabValido(self::VERSAO_ARQUIVO_LOTE);

        // CAMPO: CNAB
        // POSIÇÃO: 017 a 017
        // PICTURE: 9(1)
        //Defaul: Brancos
        $arquivo .= self::preenche_brancos('', 1);

        // CAMPO: Tipo Inscrição 
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
        if($cdfilial){
            $filial = Filial::findOneByCdfilial($cdfilial);
            if($filial->getIdtpijurfili() == 'J'){
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else {
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }else {
            if (strlen($idtpijurempr) == '14'){
              $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else{
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }
        
        $arquivo .= Util::campoCnabValido($stTpInscCedente);
        
        // CAMPO: Número Inscrição
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        if($cdfilial) {
            $filial = Filial::findOneByCdfilial($cdfilial);
            if ($filial) {
                $nrinsjurempr = Util::campoCnabValido($filial->getNrinsjurfili());
            } else {
                $nrinsjurempr = Util::campoCnabValido($nrinsjurempr->getNrinsjurempr());
            }
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr, 15, "0", STR_PAD_LEFT));
        } else{
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 15, "0", STR_PAD_LEFT));
        }
        
        // CAMPO: Convênio
        // POSIÇÃO: 034 a 053
        // PICTURE: X(20)
        //Banrisul: o código do beneficiário possui 13 dígitos (cedente 13 + 7 zeros)
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr->getCdcedenteban()){
            $arquivo .= str_pad($contcorr->getCdcedenteban(), 13, '0', STR_PAD_LEFT);
            $arquivo .= str_repeat('0',7);
        }else{
            $arquivo .= str_repeat('0',20);
        }
        
        // CAMPO: Agência Conta Corrente
        // POSIÇÃO: 054 a 058
        // PICTURE: 9(5)
        //Defaul: Brancos
        
        // CAMPO: DV
        // POSIÇÃO: 059 a 059
        // PICTURE: X(1)
        //Defaul: Brancos
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia()){
            $banco = Banco::findOneByCdbancoAndNrorg($cdbanco, $nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            if($digitoAgencia){
              $arquivo.= str_pad(substr($contcorr->getCdagencia(),0,5),6,'0',STR_PAD_LEFT);
            } else {
              $arquivo.= str_pad(str_pad(substr($contcorr->getCdagencia(),0,5),5,'0',STR_PAD_LEFT),6,'0',STR_PAD_RIGHT);
            }
        }else{
          $arquivo .= self::preenche_brancos('', 6);
        }
        
        // CAMPO: Número Conta
        // POSIÇÃO: 060 a 071
        // PICTURE: 9(12)
        //Defaul: Brancos
        
        // CAMPO: DV
        // POSIÇÃO: 072 a 072
        // PICTURE: X(1)
        //Defaul: Brancos
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
            
        } else {
            $arquivo .= str_repeat('0',12);
        }
        
        // CAMPO: DV
        // POSIÇÃO: 073 a 073
        // PICTURE: X(1)
        //Defaul: Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Nome da empresa
        // POSIÇÃO: 074 a 103
        // PICTURE: X(30)
        $nomeempresa = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        
        // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o nome da filial e não da empresa.
        if($cdfilial) {
            $filial = Filial::findOneByCdfilial($cdfilial);
            if($filial) {
                $nmempresa = substr($filial->getNmrazsocfili(), 0, 30);
            } else {
                $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
            }
            $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
        } else {
            $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
            $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
        }
        
        // CAMPO: Informação 1
        // POSIÇÃO: 104 a 143
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        
        // CAMPO: Informação 2
        // POSIÇÃO: 144 a 183
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        
        // CAMPO: Nº Remessa/Retorno
        // POSIÇÃO: 184 a 191
        // PICTURE: 9(8)
        $arquivo .= Util::campoCnabValido(substr($nrcnabrec, -8));
        
        // CAMPO: Data Gravação
        // POSIÇÃO: 191 a 199
        // PICTURE: 9(8)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
        $arquivo .= Util::campoCnabValido($data);
      
        // CAMPO: Data do crédito
        // POSIÇÃO: 200 a 207
        // PICTURE: 9(8)
        //Defaul: Brancos
        $arquivo .= self::preenche_brancos('', 8);
        
        // CAMPO: CNAB 
        // POSIÇÃO: 208 a 240
        // PICTURE: X(33)
        //Defaul: Brancos
        $arquivo = str_pad($arquivo, 240) . "\r\n";;
        
        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Detalhe Segmento P
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdagenciacnab
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $nossonumero
     * @param $idtipocobranca
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $dtvenbolrec
     * @param $valorfinalboleto
     * @param $valorpemobrec
     * @param $valormordiabrec
     * @param $valorpemulbrec
     * @param $valormultabrec
     * @param $valorfinalbrec
     * @param $cdcartcobbol
     * @param $cddiaprotbol
     * @param $cdreducont
     * @param $imprbol
     * @param $qtdiaprotbol
     * @param $contador
     * @return mixed|void
     */
    public function geraDetalheSegmentoP(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec,
        $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero, $idtipocobranca,
        $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec, $valorpemulbrec,
        $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol, $qtdiaprotbol, $contador = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Banco 
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defaul: 041
        /**@var $banco Banco*/
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();

        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Defaul: 0001 a 9999
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);

        // CAMPO: Registro 
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Defaul: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
        
        // CAMPO: Nº do registro  
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Defaul: 00001 a 99999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento 
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Defaul: P
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_P);
        
        // CAMPO: CNAB 
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Defaul: Branco
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento 
        // POSIÇÃO: 016 a 017
        // PICTURE: X(2)
        //Defaul: Branco
        $idopecnabrec = $idoperacao;
        if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= self::preenche_brancos(self::ID_OPERACAO_CNAB_EXCLUSAO);//Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
        }
        
        // CAMPO: Código Agencia 
        // POSIÇÃO: 018 a 022
        // PICTURE: 9(5)
        //Defaul: Branco
        
        // CAMPO: DV Agencia 
        // POSIÇÃO: 023 a 023
        // PICTURE: X(1)
        //Defaul: Branco
        
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia()){
            $banco = Banco::findOneByCdbancoAndNrorg($cdbanco, $nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            if($digitoAgencia){
              $arquivo.= str_pad(substr($contcorr->getCdagencia(),0,5),6,'0',STR_PAD_LEFT);
            } else {
              $arquivo.= str_pad(str_pad(substr($contcorr->getCdagencia(),0,5),5,'0',STR_PAD_LEFT),6,'0',STR_PAD_RIGHT);
            }
        }else{
          $arquivo .= self::preenche_brancos('', 6);
        }
        
        // CAMPO: Número Conta
        // POSIÇÃO: 024 a 035
        // PICTURE: 9(12)
        //Defaul: Branco
        
        // CAMPO: DV Conta 
        // POSIÇÃO: 036 a 036
        // PICTURE: X(1)
        //Defaul: Branco
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
            
        } else {
            $arquivo .= str_repeat('0',12);
        }
        
        // CAMPO: DV Conta 
        // POSIÇÃO: 037 a 037
        // PICTURE: X(1)
        //Defaul: Branco
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Nosso Número 
        // POSIÇÃO: 038 a 057
        // PICTURE: 9(20)
        //Defaul: Branco
        /*Banrisul: o Banrisul utiliza o nosso número com 10 dígitos, sendo os 8 primeiros de escolha livre do
            beneficiário e os 2 últimos calculados (o penúltimo pelo módulo 10 e o último pelo módulo 11).
            Preenchimento obrigatório quando o boleto for emitido pelo beneficiário (campo 17.3P = 2) ou quando o
            código de movimento (campo 07.3P) for diferente de 01. Informar o nosso número nas posições 38-47 e
            deixar em branco as posições 48-57.
        */
        if($imprbol == 'B'){
            $arquivo .= self::preenche_brancos('', 20);
        } else {
            $arquivo .= str_pad(str_replace([' ', '-'], '',Util::campoCnabValido($nossonumero)), 10, '0', STR_PAD_LEFT);
            $arquivo .= self::preenche_brancos('', 10);
        }
        
        // CAMPO: Carteira 
        // POSIÇÃO: 058 a 058
        // PICTURE: 9(1)
        /* '1' = Cobrança Simples
           '2' = Cobrança Vinculada 
           '3' = Cobrança Caucionada
           '4' = Cobrança Descontada
           ‘5’ = Cobrança Vendor (não tratado)
        */
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        $arquivo .= $boleto->getIdcartcobrec();
        
        if($boleto->getIdcartcobrec() === '1'){
            $this->inQtCobrSimples++;
            $this->vrCobrSimples +=  $boleto->getVrfinalbrec();
        }elseif($boleto->getIdcartcobrec() === '2'){
            $this->inQtCobrVinc++;
            $this->vrCobrVinc +=  $boleto->getVrfinalbrec();
        }elseif($boleto->getIdcartcobrec() === '3'){
            $this->inQtCobrCalc++;
            $this->inQtCobrDesc++;
            $this->vrCobrCalc +=  $boleto->getVrfinalbrec();
            $this->vrCobrDesc +=  $boleto->getVrfinalbrec();
        }
        // CAMPO: Cadastramento  
        // POSIÇÃO: 059 a 059
        // PICTURE: 9(1)
        // 1- Com Cadastramento(Cobrança Registrada) / 2 Sem Cadastramento/ 3 Com Cadastramento/Recusa do Débito
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        if($boleto && $boleto->getIdformcadbrec())
            $arquivo .= $boleto->getIdformcadbrec();
        else
            $arquivo .= '1';
        
        // CAMPO: Documento
        // POSIÇÃO: 060 a 060
        // PICTURE: X(1)
        //'1' = Tradicional; '2' = Escritural
        $arquivo .= self::TIPO_DOCUMENTO;
        
        // CAMPO: Emissão bloqueto
        // POSIÇÃO: 061 a 061
        // PICTURE: 9(1)
        // 1- BANCO EMITE/ 2 CLIENTE EMITE
        if($imprbol == 'B')
          $arquivo .= '1';
         else
         $arquivo .= '2';
          
        // CAMPO: Distrib. bloqueto
        // POSIÇÃO: 062 a 062
        // PICTURE: X(1)
        // 1- BANCO DISTRIBUI/ 2 CLIENTE DISTRIBUI
        if($imprbol == 'B')
          $arquivo .= '1';
         else
         $arquivo .= '2';
         
        // CAMPO: Nº do documento
        // POSIÇÃO: 063 a 077
        // PICTURE: X(15)
        $entityBoleto = Boletreceber::getRepository();
        $duplicata = $entityBoleto->getDuplicata($cdfilial, $nrlanbolrec);
        if (!$duplicata[0]['DUPLICATA']){
            $arquivo .= str_pad($nrlanbolrec, 10, '0');
        }else {
            $arquivo .= str_pad(substr($duplicata[0]['DUPLICATA'], -8), 8, '0', STR_PAD_LEFT)  . str_pad($duplicata[0]['NRPARCELA'], 2, '0', STR_PAD_LEFT);
            $arquivo .= self::preenche_brancos('', 5);
        }
        
        // CAMPO: Vencimento
        // POSIÇÃO: 078 a 085
        // PICTURE: 9(8)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $vencimento = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
        $arquivo .= Util::campoCnabValido($vencimento);
        
        // CAMPO: Valor do título
        // POSIÇÃO: 086 a 100
        // PICTURE: 9(15)
        $vrfinabolrec = str_pad(($valorfinalboleto * 100), 15, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vrfinabolrec);
        
        // CAMPO: Agência cobradora
        // POSIÇÃO: 101 a 105
        // PICTURE: 9(5)
        //Deault: Brancos
        $cdagencia = str_pad($cdagenciacnab, 5);
        if($cddiaprotbol != '3')
            $arquivo = str_pad($arquivo, 105, "0");

        elseif((bool) $cdagencia !== false)
            $arquivo .= Util::campoCnabValido($cdagencia);

        else
           $arquivo = str_pad($arquivo, 105, "0");
        
        // CAMPO: DV
        // POSIÇÃO: 106 a 106
        // PICTURE: X(1)
        //Deault: Brancos
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Espécie título
        // POSIÇÃO: 107 a 108
        // PICTURE: 9(2)
        // '02' = DM Duplicata Mercantil
        $arquivo .= Util::campoCnabValido(self::ESPECIE_TITULO);
        
        // CAMPO: Aceite
        // POSIÇÃO: 109 a 109
        // PICTURE: X(1)
        //A' = Aceite; 'N' = Não Aceite
        $arquivo .= Util::campoCnabValido(self::IDTITULO_ACEITO);
        
        // CAMPO: Data Emissão Título
        // POSIÇÃO: 110 a 117
        // PICTURE: 9(8)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Cód. Juros de Mora
        // POSIÇÃO: 118 a 118
        // PICTURE: 9(1)
        /**'1' = Valor por Dia 
         * '2' = Taxa Mensal 
         * '3' = Isento 
         * Banrisul: para cobrança descontada (14.3P = 4), preencher com '3'.
         * **/
         
        // CAMPO: Data Juros Mora
        // POSIÇÃO: 119 a 126
        // PICTURE: 9(8)
        
        // CAMPO: Juros Mora
        // POSIÇÃO: 127 a 141
        // PICTURE: 9(15)
        
        $vrpemorbrec = $valorpemobrec;
        $vrmordiabrec = $valormordiabrec;
        $vrpemulbrec = $valorpemulbrec;
        $vrmultabrec = $valormultabrec;
        
        if ((($vrpemorbrec == 0) or ((bool) $vrpemorbrec == false)) and (($vrmordiabrec == 0) or ((bool) $vrmordiabrec == false))){
            $arquivo .= Util::campoCnabValido('3');
            $arquivo = str_pad($arquivo, 126, "0");
            $arquivo = str_pad($arquivo, 141, "0");
        }else{
            if(($vrpemorbrec != 0) or ((bool) $vrpemorbrec == true)){
              $arquivo .= Util::campoCnabValido('2');
              $arquivo .= Util::campoCnabValido($vencimento);
              $vr = $vrpemorbrec * 100;
              $vr = str_pad($vr, 15, '0', STR_PAD_LEFT);
              $arquivo .= Util::campoCnabValido($vr);
            } else {
              $arquivo .= Util::campoCnabValido('1');
              $arquivo .= Util::campoCnabValido($vencimento);
              $vr = $vrmordiabrec * 100;
              $vr = str_pad($vr, 15, '0', STR_PAD_LEFT);
              $arquivo .= Util::campoCnabValido($vr);
            }

        }
        
        // CAMPO: Cód Desconto 1
        // POSIÇÃO: 142 a 142
        // PICTURE: 9(1)
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Data Desconto 1
        // POSIÇÃO: 143 a 150
        // PICTURE: 9(8)
        $arquivo .= str_repeat('0',8);
        
        // CAMPO: Desconto 1
        // POSIÇÃO: 151 a 165
        // PICTURE: 9(15)
        $arquivo .= str_repeat('0',15);
        
        // CAMPO: Valor IOF
        // POSIÇÃO: 166 a 180
        // PICTURE: 9(15)
        $arquivo .= str_repeat('0',15);
        
        // CAMPO: Valor Abatimento
        // POSIÇÃO: 181 a 195
        // PICTURE: 9(15)
        $arquivo .= str_repeat('0',15);
        
        // CAMPO: Uso Empresa Beneficiária
        // POSIÇÃO: 196 a 220
        // PICTURE: X(25)
        $arquivo .= str_pad($cdfilial.$nrlanbolrec,25);
        
        // CAMPO: Código para Protesto
        // POSIÇÃO: 221 a 221
        // PICTURE: 9(1)
        /** '1' = Protestar Dias Corridos
         * '3' = Não Protestar
         * '9' = Cancelamento Protesto Automático (somente válido p/ CódigoMovimento Remessa = '31' - Ver C004)
        */
        if($cddiaprotbol){
            if($cddiaprotbol == '1'){
                $arquivo .= '1';
            } elseif($cddiaprotbol == '3'){
                $arquivo .= '3';
            }
        } else{
            $arquivo .= '3';
        }
        
        // CAMPO: Prazo para Protesto
        // POSIÇÃO: 222 a 223
        // PICTURE: 9(2)
        if($qtdiaprotbol){
            $arquivo .= str_pad(Util::campoCnabValido($qtdiaprotbol), 2, "0", STR_PAD_LEFT);
            
        } else{
            $arquivo .= '00';
        }
        
        // CAMPO: Cód para Baixa/Devolução
        // POSIÇÃO: 224 a 224
        // PICTURE: 9(1)
        /* *'1' = Baixar / Devolver
         * '3' = Cancelar Prazo para Baixa/Devolução (somente válido p/ Código Movimento Remessa = '31' - Ver C004)
         */
         $arquivo .= Util::campoCnabValido(self::CODIGO_BAIXA);
        
        // CAMPO: Prazo para Baixa/ Devolução
        // POSIÇÃO: 225 a 227
        // PICTURE: X(3)
        $arquivo .= str_repeat('0',3);
        
        // CAMPO: Código da Moeda
        // POSIÇÃO: 228 a 229
        // PICTURE: 9(2)
        // '09' = Real
        $arquivo .= Util::campoCnabValido(self::CODIGO_MOEDA);
        
        // CAMPO: Código de espécie de cobrança
        // POSIÇÃO: 230 a 239
        // PICTURE: 9(10)
        /*
         * CÓDIGO DE CARTEIRA  = 1 
         * 805076=Cobrança simples com registro 
         * 805157=Cobrança simples seguradoras
         * 805238=Cobrança simples financeiras
         * 815470=Cobrança Partilhada
         * 825468=Cobrança simples dólar
         * CÓDIGO DE CARTEIRA  = 4
         * 603015=Desconto de títulos
         **/
         $arquivo .= str_pad($contcorr->getCdidreccontcorr(),10);
         
        // CAMPO: Autorização de pagamento parcial
        // POSIÇÃO: 340 a 240
        // PICTURE: x(10)
        /**
         * Uso livre banco/empresa - 1 NÃO AUTORIZA / 2 AUTORIZA
         * Banrisul: para o código 2, é obrigatório incluir segmento Y-53. Para cobrança descontada (campo 14.3P = 4), 
            preencher com '1'.
        */
        $arquivo .= Util::campoCnabValido(self::PGTOPARCIAL);
        
        
        // CAMPO: Reservado (uso Banco)
        // POSIÇÃO: 230 -240
        // PICTURE: 011
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Detalhe  Segmento Q
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdcliente
     * @param $idoperacao
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoQ(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL
    ){

       
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
        
        // CAMPO: Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Default: 041
        /**@var $banco Banco*/
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Default: 0001 a 9999
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
         
        // CAMPO: Nº do registro
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Default: 0001 a 9999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Default: Q
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_Q);
        
        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Default: BRANCOS
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        $idopecnabrec = $idoperacao;
        if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= self::preenche_brancos(self::ID_OPERACAO_CNAB_EXCLUSAO);//Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
        }
        
        // CAMPO: Tipo de inscrição
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
        $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
        if($cdfilial){
            $filial = Filial::findOneByCdfilial($cdfilial);
            if($filial->getIdtpijurfili() == 'J'){
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else {
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }else {
            if (strlen($idtpijurempr) == '14'){
              $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
            } else{
                $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
            }
        }
        
        $arquivo .= Util::campoCnabValido($stTpInscCedente);
        
        // CAMPO: Número de inscrição
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o NRINSJUR da filial e não da empresa.
        if($cdfilial) {
            $filial = Filial::findOneByCdfilial($cdfilial);
            if ($filial) {
                $nrinsjurempr = Util::campoCnabValido($filial->getNrinsjurfili());
            } else {
                $nrinsjurempr = Util::campoCnabValido($nrinsjurempr->getNrinsjurempr());
            }
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr, 15, "0", STR_PAD_LEFT));
        } else{
            $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 15, "0", STR_PAD_LEFT));
        }
        
        // CAMPO: Nome
        // POSIÇÃO: 034 a 073
        // PICTURE: X(40)
        $cliente = Cliente::findOneByNrorgAndCdcliente($nrorg, $cdcliente);
        if($cdcliente){
            $copia_nmrazsoc = $cliente->getNmrazsocclie();
            $nmrazsocclie = Util::campoCnabValido($copia_nmrazsoc);
            $arquivo .= str_pad(substr($nmrazsocclie, 0, 40) ,40);
        }
        else{
            $arquivo .= self::preenche_brancos('', 40);
        }
        
        // CAMPO: Endereço
        // POSIÇÃO: 074 a 113
        // PICTURE: X(40)
        $endeclie = Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie($nrorg, $cdcliente, 'C');
        $dsendeclie = Util::campoCnabValido($endeclie->getDsendeclie());
        if((bool) $dsendeclie !== false){
            $arquivo .= str_pad(substr($dsendeclie, 0, 40), 40);
        }
        else{
          $arquivo .= self::preenche_brancos('', 40);
        }
        
        // CAMPO: Bairro
        // POSIÇÃO: 114 a 128
        // PICTURE: X(15)
        //DEFAULT: BRANCOS
        $nmbairclie = str_pad(Util::campoCnabValido(trim($endeclie->getNmbairclie())), 15);
        if((bool) $nmbairclie !== false){
           $arquivo .= substr($nmbairclie, 0, 15);
        }
        else{
          $arquivo .= self::preenche_brancos('', 15);
        }
        
        // CAMPO: CEP
        // POSIÇÃO: 129 a 133
        // PICTURE: 9(5)
        //DEFAULT: BRANCOS
        
        // CAMPO: Sufixo do CEP
        // POSIÇÃO: 134 a 136
        // PICTURE: 9(3)
        $nrcepclie = str_pad(str_replace(" ", null, $endeclie->getNrcepclie()), 8, "0");
        if((bool) $nrcepclie !== false){
            $arquivo .= Util::campoCepTel($nrcepclie);
        }
        else{
           $arquivo .= str_repeat('0',8);
        }
        
        // CAMPO: Cidade
        // POSIÇÃO: 137 a 151
        // PICTURE: X(15)
        $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $endeclie->getCdmunicipio());
        $nmmunicipio = substr($municipio->getNmmunicipio(), 0, 15);
        if((bool) $nmmunicipio !== false){
          $arquivo .= Util::campoCnabValido(str_pad($nmmunicipio, 15));
        }
        else{
          $arquivo .= self::preenche_brancos('', 15);
        }
        
        // CAMPO: UF
        // POSIÇÃO: 152 a 153
        // PICTURE: X(2)
        $sgestado = str_pad($endeclie->getSgestado(), 2);
        if((bool) $sgestado !== false){
           $arquivo .= Util::campoCnabValido($sgestado);
        }
        else{
           $arquivo .= self::preenche_brancos('', 2);
        }
        
        // CAMPO: Tipo de inscrição
        // POSIÇÃO: 154 a 154
        // PICTURE: 9(1)
        //DEFAULT: BRANCOS ou 0
        $arquivo .= str_repeat('0',1);
        
        // CAMPO: Número de inscrição
        // POSIÇÃO: 155 a 169
        // PICTURE: 9(15)
        //DEFAULT: BRANCOS
        $arquivo .= str_repeat('0',15);
        
        // CAMPO: Nome do sacador/ avalista
        // POSIÇÃO: 170 a 209
        // PICTURE: X(40) 
        //DEFAULT: BRANCOS
        $arquivo .= self::preenche_brancos('', 40);
        
        // CAMPO: Banco Correspondente
        // POSIÇÃO: 210 a 212
        // PICTURE: 9(3)
        //DEFAULT: BRANCOS   
        $arquivo .= str_repeat('0',3);
        
        // CAMPO: Nosso Número
        // POSIÇÃO: 213 a 232
        // PICTURE: X(20)
        //DEFAULT: BRANCOS 
        
         $arquivo .= self::preenche_brancos('', 20);
        // CAMPO: CNAB
        // POSIÇÃO: 233 a 240
        // PICTURE: X(8)
        //DEFAULT: BRANCOS   
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Detalhe Segmento R
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoR(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        // DEFALT: 041
        /**@var $banco Banco */
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();

        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);

        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);

        // CAMPO: Nº seq. do reg. no arq.
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);

        // CAMPO: Código do Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: 9(1)
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_R);

        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        $arquivo .= self::preenche_brancos('', 1);

        // CAMPO: Código de Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        $idopecnabrec = $idoperacao;
        if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= self::preenche_brancos(self::ID_OPERACAO_CNAB_EXCLUSAO);//Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
        }

        // CAMPO: Código de desconto 2
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $arquivo .= Util::campoCnabValido(self::CODIGO_DESCONTO_2);

        // CAMPO: Data do desconto 2
        // POSIÇÃO: 019 a 026
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 26, "0");

        // CAMPO: Desconto 2
        // POSIÇÃO: 027 a 041
        // PICTURE: 9(15)
        $arquivo = str_pad($arquivo, 41, "0");

        // CAMPO: Código de desconto 3
        // POSIÇÃO: 042 a 042
        // PICTURE: 9(1)
        $arquivo = str_pad($arquivo, 42, "0");
        
        // CAMPO: Data Desc 3
        // POSIÇÃO: 043 a 050
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 50, "0");
        
        // CAMPO: Desconto 3
        // POSIÇÃO: 051 a 065
        // PICTURE: 9(15)
        $arquivo = str_pad($arquivo, 65, "0");
        
        // CAMPO: Cód Multa
        // POSIÇÃO: 066 a 066
        // PICTURE: X(1)
        $vrpemulbrec = $valorpemulbrec;
        if($vrpemulbrec == 0)
            $arquivo .= Util::campoCnabValido(self::CODIGO_MULTA_1);
        else
            $arquivo .= Util::campoCnabValido(self::CODIGO_MULTA_2);
        
        // CAMPO: Data da Multa
        // POSIÇÃO: 067 a 074
        // PICTURE: 9(8)
        $now  = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $tomorrow = DateUtil::truncateString(DateUtil::adicionaIntervalo(DateUtil::truncateData($now), 1, 'D'));
        $data = DateUtil::getDia($tomorrow) . DateUtil::getMes($tomorrow) . DateUtil::getAno($tomorrow);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Multa
        // POSIÇÃO: 075 a 089
        // PICTURE: 9(15)
        $vrmultabrec = $valormultabrec;
        if (($vrpemulbrec == 0) or ((bool) $vrpemulbrec === false)){
            if (($vrmultabrec == 0) or ((bool) $vrmultabrec === false)){
                $arquivo = str_pad($arquivo, 89, "0");
            }
            else{
                $vrmultabrec = $vrmultabrec * 100;
                $arquivo .= Util::campoCnabValido(str_pad($vrmultabrec, 15, '0', STR_PAD_LEFT));
            }
        }
        else{
            $vrpemulbrec = $vrpemulbrec * 100;
            $arquivo .= Util::campoCnabValido(str_pad($vrpemulbrec, 15, '0', STR_PAD_LEFT));
        }
        
        // CAMPO: Informação ao Pagador
        // POSIÇÃO: 090 a 099
        // PICTURE: 9(10)
        $arquivo .= self::preenche_brancos('', 10);
        
        // CAMPO: Informação 3
        // POSIÇÃO: 100 a 139
        // PICTURE: X(40)
        $boletreceber = Boletreceber::findOneByNrorgAndNrlanbolrec($nrorg, $nrlanbolrec);
        if((bool)$boletreceber->getTxmensbolbol() === false){
            $arquivo = str_pad($arquivo, 139);
        }
        else{
            $arquivo .= Util::campoCnabValido($boletreceber->getTxmensbolbol());
        }
        
        // CAMPO: Informação 4
        // POSIÇÃO: 140 a 179
        // PICTURE: X(40)     
        $arquivo = str_pad($arquivo, 179);
        
        // CAMPO: CNAB
        // POSIÇÃO: 180 a 199
        // PICTURE: X(20)        
        $arquivo = str_pad($arquivo, 199);
        
        // CAMPO: Cód Ocor Pagador
        // POSIÇÃO: 200 a 207
        // PICTURE: 9(8)        
        $arquivo = str_pad($arquivo, 207);
        
        // CAMPO: Banco
        // POSIÇÃO: 208 a 210
        // PICTURE: 9(3)  
        $arquivo = str_pad($arquivo, 210);
        
        // CAMPO: Agência
        // POSIÇÃO: 211 a 215
        // PICTURE: 9(5) 
        $arquivo = str_pad($arquivo, 215);
        
        // CAMPO: DV
        // POSIÇÃO: 216 a 216
        // PICTURE: X(1)
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Conta Corrente
        // POSIÇÃO: 217 a 228
        // PICTURE: 9(12)
        $arquivo = str_pad($arquivo, 228);
        
        // CAMPO: DV
        // POSIÇÃO: 229 a 229
        // PICTURE: X(1)
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: DV AG/CONTA
        // POSIÇÃO: 230 a 230
        // PICTURE: X(1)
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Ident da Emissão Aviso Déb
        // POSIÇÃO: 231 a 231
        // PICTURE: 9(1)
        $arquivo .= '02';
        
        // CAMPO: CNAB
        // POSIÇÃO: 232 a 240
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    /**
     * OPCIONAL, AINDA NÃO FOI IMPLEMENTADO
     * Gera Detalhe Segmento S
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoS(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
        // CAMPO: CNAB
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defalt: 041
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Lote de serviço. É sempre igual ao do registro 1.
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
        
        // CAMPO: Nº do registro
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Default: 00001 a 99999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Default: S
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_S);
        
        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if($idopecnabrec === 'I'){
            $arquivo .= self::geraDetalheImpressaoTipo1();
        }else {
            $arquivo .= self::geraDetalheImpressaoTipo2();
        }
        // CAMPO: CNAB
        // POSIÇÃO: 232 a 240
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    public function geraDetalheImpressaoTipo1(){
        
        // CAMPO: Tipo de Impressão
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        /**
         * '1' = Frente do Boleto de Pagamento 
         * '2' = Verso do Boleto de Pagamento 
         * '3' = Corpo de Instruções da Ficha de Compensação do Boleto de Pagamento
         * */
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Número da Linha
        // POSIÇÃO: 019 a 20
        // PICTURE: 9(2)
        /**Frente do Boleto de Pagamento = de '01' à '36' 
         * Verso do Boleto de Pagamento = de '01' à '24'
         * 
         */
        $arquivo .= self::preenche_brancos('', 2);
        // CAMPO: Mensagem
        // POSIÇÃO: 21 a 160
        // PICTURE: X(140)
        /**
         * Texto de mensagem do Beneficiário destinada ao Pagador para impressão no título de cobrança. 
         * Esta linha deverá ser enviada no formato imagem de impressão (tamanho máximo de 91 posições).
         * **/
        $arquivo .= self::preenche_brancos('', 140);
        
        // CAMPO: Tipo de Fonte
        // POSIÇÃO: 161 a 162
        // PICTURE: 9(2)
        //'01' = Normal
        $arquivo .= Util::campoCnabValido(self::FONTE);
        
        // CAMPO: CNAB
        // POSIÇÃO: 163 a 240
        // PICTURE: X(78)
        $arquivo .= self::preenche_brancos('', 78);
        return $arquivo."\r\n";
    }
    
    public function geraDetalheImpressaoTipo2(){
        
        // CAMPO: Tipo de Impressão
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $arquivo .= self::preenche_brancos('', 1);
        // CAMPO: Informação 5
        // POSIÇÃO: 019 a 58
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        // CAMPO: Informação 6
        // POSIÇÃO: 059 a 98
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        // CAMPO: Informação 7
        // POSIÇÃO: 099 a 138
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        // CAMPO: Informação 8
        // POSIÇÃO: 139 a 178
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        // CAMPO: Informação 9
        // POSIÇÃO: 179 a 218
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        // CAMPO: CNAB
        // POSIÇÃO: 219 a 240
        // PICTURE: X(22)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 22);
        return $linha."\r\n";
    }
    
    /**
     * Gera Detalhe Segmento Y01
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoY01(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL, $cdreducont
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
        // CAMPO: Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defalt: 041
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Lote de serviço. É sempre igual ao do registro 1.
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
        
        // CAMPO: Nº do registro
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Default: 00001 a 99999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Default: S
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_Y);
        
        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        $idopecnabrec = $idoperacao;
        if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= self::preenche_brancos(self::ID_OPERACAO_CNAB_EXCLUSAO);//Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
        }
        
        // CAMPO: Cod Reg Opcional
        // POSIÇÃO: 018 a 019
        // PICTURE: 9(2)
        //Default: 01
         $arquivo .= '01';
         
        // CAMPO: Tipo de inscrição
        // POSIÇÃO: 20 a 20
        // PICTURE: 9(1)
        //(VERIFICAR)
        
        // CAMPO: Número de inscrição do sacador/avalista
        // POSIÇÃO: 21 a 35
        // PICTURE: 9(15)
        //(VERIFICAR)
        
        // CAMPO: Nome do sacador/ avalista
        // POSIÇÃO: 36 a 75
        // PICTURE: X(40)
        if($cdcliente){
            $copia_nmrazsoc = $cliente->getNmrazsocclie();
            $nmrazsocclie = Util::campoCnabValido($copia_nmrazsoc);
            $arquivo .= str_pad(substr($nmrazsocclie, 0, 40) ,40);
        }
        else{
            $arquivo .= self::preenche_brancos('', 40);
        }
        
        // CAMPO: Endereço do sacador/ avalista
        // POSIÇÃO: 76 a 130
        // PICTURE: X(40)
        $arquivo .= self::preenche_brancos('', 40);
        
        // CAMPO: Bairro
        // POSIÇÃO: 116 a 130 
        // PICTURE: X(15)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 15);
        
        // CAMPO: Cep
        // POSIÇÃO: 131 a 135 
        // PICTURE: 9(5)
        
        // CAMPO: Sufixo do CEP
        // POSIÇÃO: 136 a 138 
        // PICTURE: 9(3)
        $arquivo .= str_repeat('0',8);
        
        // CAMPO: Cidade
        // POSIÇÃO: 139 a 153 
        // PICTURE: X(15)
        $arquivo .= self::preenche_brancos('', 15);
        
        // CAMPO: UF
        // POSIÇÃO: 154 a 155 
        // PICTURE: X(2)
        $arquivo .= self::preenche_brancos('', 2);

        // CAMPO: CNAB
        // POSIÇÃO: 156 a 240 
        // PICTURE: X(85)
        //Default: Brancos
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    /**
     * * OPCIONAL, AINDA NÃO FOI IMPLEMENTADO
     * Gera Detalhe Segmento Y50
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoY50(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
        // CAMPO: Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defalt: 041
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Lote de serviço. É sempre igual ao do registro 1.
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
        
        // CAMPO: Nº do registro
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Default: 00001 a 99999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Default: S
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_Y);
        
        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if($idopecnabrec === 'I'){
            $arquivo .= self::geraDetalheImpressaoTipo1();
        }else {
            $arquivo .= self::geraDetalheImpressaoTipo2();
        }
        
        // CAMPO: Cod Reg Opcional
        // POSIÇÃO: 018 a 019
        // PICTURE: 9(2)
        //Default: 50
         $arquivo .= '50';
         
        // CAMPO: Agência mantenedora da conta
        // POSIÇÃO: 020 a 024
        // PICTURE: 9(5)
        
        // CAMPO: DV
        // POSIÇÃO: 025 a 025
        // PICTURE: X(1)
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia()){
            $banco = Banco::findOneByCdbancoAndNrorg($cdbanco, $nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            if($digitoAgencia){
              $arquivo.= str_pad(substr($contcorr->getCdagencia(),0,5),6,'0',STR_PAD_LEFT);
            } else {
              $arquivo.= str_pad(str_pad(substr($contcorr->getCdagencia(),0,5),5,'0',STR_PAD_LEFT),6,'0',STR_PAD_RIGHT);
            }
        }else{
          $arquivo .= self::preenche_brancos('', 6);
        }
        
        // CAMPO: Número da conta corrente
        // POSIÇÃO: 026 a 037
        // PICTURE: 9(12)
        
        // CAMPO: DV
        // POSIÇÃO: 038 a 038
        // PICTURE: X(1)
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
            
        } else {
            $arquivo .= self::preenche_brancos('', 12);
        }
         
        if($contcorr) {
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
        } else {
            $arquivo .= self::preenche_brancos('', 1);
        }
        
        // CAMPO: DV AG/CC
        // POSIÇÃO: 039 a 039
        // PICTURE: X(1)
        //Default Brancos
        $arquivo .= self::preenche_brancos('', 1);
         
        // CAMPO: Nosso Número
        // POSIÇÃO: 040 a 059
        // PICTURE: 9(20)       
        if($imprbol == 'B'){
            $arquivo .= self::preenche_brancos('', 20);
        } else {
            $arquivo .= str_pad($nossonumero, 20, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Cod Cálc Rateio p/ benef
        // POSIÇÃO: 060 a 060
        // PICTURE: 9(1)       
        
        // CAMPO: Tipo de Valor Informado
        // POSIÇÃO: 061 a 061
        // PICTURE: 9(1)       
        
        // CAMPO: Valor ou % (Percentual) 
        // POSIÇÃO: 062 a 076
        // PICTURE: 9(15)       
        
        // CAMPO: % (Percentual)
        // POSIÇÃO: 062 a 076
        // PICTURE: 9(15)       
        
        // CAMPO: Código do Banco
        // POSIÇÃO: 077 a 079
        // PICTURE: 9(3)       
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Agência mantenedora da conta
        // POSIÇÃO: 080 a 084
        // PICTURE: 9(5)       
        
        // CAMPO: Dígito verificador da agência
        // POSIÇÃO: 085 a 085
        // PICTURE: X(1)
        //Default Brancos
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia()){
            $banco = Banco::findOneByCdbancoAndNrorg($cdbanco, $nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
            if($digitoAgencia){
              $arquivo.= str_pad(substr($contcorr->getCdagencia(),0,5),6,'0',STR_PAD_LEFT);
            } else {
              $arquivo.= str_pad(str_pad(substr($contcorr->getCdagencia(),0,5),5,'0',STR_PAD_LEFT),6,'0',STR_PAD_RIGHT);
            }
        }else{
          $arquivo .= self::preenche_brancos('', 6);
        }
        
        // CAMPO: Número da conta corrente
        // POSIÇÃO: 086 a 097
        // PICTURE: 9(12)
        
        // CAMPO: Dígito verificador da conta
        // POSIÇÃO: 098 a 098
        // PICTURE: X(1)
        //Default Brancos
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
            
        } else {
            $arquivo .= self::preenche_brancos('', 12);
        }
         
        if($contcorr) {
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
        } else {
            $arquivo .= self::preenche_brancos('', 1);
        }
        
        // CAMPO: Dígito verificador da conta
        // POSIÇÃO: 098 a 098
        // PICTURE: X(1)
        //Default Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: CNAB
        // POSIÇÃO: 232 a 240
        // PICTURE: X(9)
        //Default: Brancos
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    /**
     * OPCIONAL, AINDA NÃO FOI IMPLEMENTADO
     * Gera Detalhe Segmento Y53
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoY53(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
        // CAMPO: Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Defalt: 041
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();
        
        // CAMPO: Lote
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Lote de serviço. É sempre igual ao do registro 1.
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
        
        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 3
        $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTO);
        
        // CAMPO: Nº do registro
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        //Default: 00001 a 99999
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);
        
        // CAMPO: Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        //Default: S
        $arquivo .= Util::campoCnabValido(self::SEGMENTO_Y);
        
        // CAMPO: CNAB
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Cód de movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        if($idopecnabrec === 'I'){
            $arquivo .= self::geraDetalheImpressaoTipo1();
        }else {
            $arquivo .= self::geraDetalheImpressaoTipo2();
        }
        
        // CAMPO: Cod Reg Opcional
        // POSIÇÃO: 018 a 019
        // PICTURE: 9(2)
        //Default: 53
         $arquivo .= '53';
         
        // CAMPO: Identificação de Tipo de pagamento
        // POSIÇÃO: 020 a 021
        // PICTURE: 9(2)
         
        // CAMPO: Quantidade de pagamentos possíveis
        // POSIÇÃO: 022 a 023
        // PICTURE: 9(2)
         
        // CAMPO: Tipo de valor
        // POSIÇÃO: 024 a 024
        // PICTURE: 9(1)
         
        // CAMPO: Valor máximo/Percentual
        // POSIÇÃO: 025 a 039
        // PICTURE: 9(15)
        
        // CAMPO: Tipo de valor
        // POSIÇÃO: 040 a 040
        // PICTURE: 9(1)
         
        // CAMPO: Valor máximo/Percentual
        // POSIÇÃO: 041 a 055
        // PICTURE: 9(15)
       
        // CAMPO: CNAB
        // POSIÇÃO: 055 a 240
        // PICTURE: X(185)
        //Default: Brancos
        $arquivo = str_pad($arquivo, 240) . "\r\n";
        
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Trailler Lote
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param null $contreg
     * @return mixed|void
     */
    public function geraTraillerLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL
    ){

        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        //Default: 041
        /** @var $banco Banco */
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        //Default: Inicio 0001 a 9999
        $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);

        // CAMPO: Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 5
        $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO_TRAILLER_LOTE);

        // CAMPO: CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 9);

        // CAMPO: Quantidade de registros do arquivo
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contreg - 1, 6, "0", STR_PAD_LEFT));
        
        // CAMPO: Totalização da Cobrança Simples- Quantidade
        // POSIÇÃO: 024 a 029
        // PICTURE: 9(6)
        $arquivo .= str_pad($this->inQtCobrSimples, 6, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Simples - Valor
        // POSIÇÃO: 030 a 046
        // PICTURE: 9(17)
        $arquivo .= str_pad($this->vrCobrSimples, 17, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Vinculada- Quantidade
        // POSIÇÃO: 047 a 052
        // PICTURE: 9(6)
        $arquivo .= str_pad($this->inQtCobrVinc, 6, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Vinculada - Valor
        // POSIÇÃO: 053 a 069
        // PICTURE: 9(17)
        $arquivo .= str_pad($this->vrCobrVinc, 17, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Caucionada- Quantidade
        // POSIÇÃO: 070 a 075
        // PICTURE: 9(6)
        $arquivo .= str_pad($this->inQtCobrCalc, 6, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Caucionada - Valor
        // POSIÇÃO: 076 a 092
        // PICTURE: 9(17)
        $arquivo .= str_pad($this->vrCobrCalc, 17, "0", STR_PAD_LEFT);
        
        // CAMPO: Totalização da Cobrança Descontada- Quantidade
        // POSIÇÃO: 093 a 098
        // PICTURE: 9(6)
        $arquivo .= str_pad($this->inQtCobrDesc, 6, "0", STR_PAD_LEFT);

        // CAMPO: Totalização da Cobrança Descontada - Valor
        // POSIÇÃO: 099 a 115
        // PICTURE: 9(17)
        $arquivo .= str_pad($this->vrCobrDesc, 6, "0", STR_PAD_LEFT);
        
        // CAMPO: Número do Aviso
        // POSIÇÃO: 116 a 123
        // PICTURE: X(8)
        $arquivo .= self::preenche_brancos('', 8);
        
        // CAMPO: CNAB
        // POSIÇÃO: 124 - 240
        // PICTURE: 9(117)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    /**
     * Gera Trailler
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * * @param null $contreg
     * @return mixed|void
     */
    public function geraTrailler(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL
    ){
        $nmarquivo = $caminho . $nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        /**@var $banco Banco */
        $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
        $arquivo = $banco->getCdbanccnab();

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= Util::campoCnabValido(self::LOTE_DO_SERVICO);

        // CAMPO: Identificação do Arquivo
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        //Default: 9
        $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO_TRAILLER);

        // CAMPO: CNAB
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        //Default: Brancos
        $arquivo .= self::preenche_brancos('', 9);

        // CAMPO: Qtde de Lotes
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contlote, 6, "0", STR_PAD_LEFT));

        // CAMPO: Qtde de Registros
        // POSIÇÃO: 024 a 029
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contreg, 6, "0", STR_PAD_LEFT));
        
        // CAMPO: Qtde de Contas Concil.
        // POSIÇÃO: 030 a 035
        // PICTURE: 9(6)
        $arquivo .= self::preenche_brancos('', 6);
        
        // CAMPO: Reservado (uso Banco)
        // POSIÇÃO: 030 - 240
        // PICTURE: 211
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
    */
     
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }

    /**
     * Gera Transação
     * @param $caminho
     * @param $nmarquivo
     * @param $cdempresa
     * @param $cdcartcobbol
     * @param $cdagencia
     * @param $cdcontcorr
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $vrpemobrec
     * @param $idimprboleto
     * @param $cdnosnumrec
     * @param $dtvenbolrec
     * @param $cdins1cobbol
     * @param $qtdiaprotbol
     * @param $vrmordiabrec
     * @param $vrfinalbrec
     * @param $idopecnabrec
     * @param $vrpemulbrec
     * @param $idtpijurclie
     * @param $nrinsjurclie
     * @param $cdcliente
     * @param $cdins2cobbol
     * @return mixed|void
     */
    public function geraTransacao(
        $caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
        $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
        $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
        $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL, $cdreducont,$cdbanco
    ){

    }
}