function Rec11003_RelatorioCustomizavelFaturamento_Controller(General, ScreenService, templateManager, ZhFilterPreferences, ZHPromise) {
    
    var self = this;
    
    self.openFilterTitulosAReceber = function(widget){
        self.buscaDadosIniciaisFiltro(widget).then(function(){
            var rowSetFilter = {
                'DTATUAVENREC' : widget.DTVENBOLREC,
                'DTEMISSAO'    : widget.DTVENBOLREC,
                'CDEMPRESA'    : [widget.empresa.CDEMPRESA]
            };
            
            ZhFilterPreferences.setOriginalFilter(widget.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowSetFilter);
        });
    };
    
    self.buscaDadosIniciaisFiltro = function(widget) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widget).then(function(){
            self.buscaPadraoIntervaloVencimento(widget).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralrec     = self.buscaGeralRec();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralrec, paramestoque]).then(function(response){
            geralrec     = response[0].dataset.geral_rec[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralrec?geralrec.QTDIAANTLANR?geralrec.QTDIAANTLANR:0:0;
            var qtdiaapolanr = geralrec?geralrec.QTDIAAPOLANR?geralrec.QTDIAAPOLANR:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLREC = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    self.buscaGeralRec = function(){
        return General.callRestEngine(
            'fin#/crud/geral_rec',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        );
    };
    
    self.buildFilter = function (widget, preference) {
        var filtro    = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;
        
        filtro.forEach(function(fil) {
            switch (fil.name) {
                case 'CDEMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_") : null;
                    break;
                case 'DTATUAVENRECINI':
                    fil.value = filterRow.DTATUAVENREC ? filterRow.DTATUAVENREC.start : null;
                    break;
                case 'DTATUAVENRECFIN':
                    fil.value = filterRow.DTATUAVENREC ? filterRow.DTATUAVENREC.end : null;
                    break;
                case 'DTEMISSAOINI':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.start : null;
                    break;
                case 'DTEMISSAOFIN':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.end : null;
                    break;
            }
        });
        
        preference.CUSTOM_FILTER_ROW = [];
    };
    
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec11003_RelatorioCustomizavelFaturamento_Controller', Rec11003_RelatorioCustomizavelFaturamento_Controller);
});