var General = function (ScreenService, templateManager, localStorageEngine, restEngine, maskEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory, MessageService) {
	var searchListDeleted;
	var parent = this;
	var cacheLoaded = false;

	//@TODO Criar objeto de mensagens para tradução.

	/*
	 * Função que atualiza o dataSourceFilter de um field que depende do resultado de
	 * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
	 * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
	 * ao alterar o valor do field "pai".
	 *
	 * @param field dependentField
	 * @param array parameters
	 *
	 * Exemplo:
	 * General.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
	 */
	this.onClickDependentField = function (dependentField, parameters) {

		var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

		var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

		if (changedDataSourceFilter) {
			changeDependetFieldFilter(dependentField, dataSourceFilterNew);
		}

		if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
			dependentField.reload();
		}
	};

	this.buildDataSourceFilter = function (dataSourceFilter, parameters) {
		if (!dataSourceFilter) {
			throw "DataSourceFilter is not defined!";
		}

		for (var position in dataSourceFilter) {
			if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
				dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
			}
		}

		return dataSourceFilter;
	};

	var changeDependetFieldFilter = function (dependentField, dataSourceFilterNew) {

		dependentField.dataSourceFilter = dataSourceFilterNew;
		dependentField.dataSource.lazyLoad = false;
		if (dependentField.selectWidget) {
			dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
		}

		if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

			searchListDeleted = true;
			delete dependentField.selectWidget.searchList;

		} else if (dependentField.selectWidget) {

			searchListDeleted = false;

		}

	};

	var compareDataSourceFilters = function (selectWidget, dataSourceFilterNew, parameters) {
		var changedDataSourceFilter = false;
		if (selectWidget) {
			var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
			for (var position = 0; position < dataSourceFilterNew.length; position++) {
				if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
					if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
						changedDataSourceFilter = true;
					}
				}
			}
		} else {
			changedDataSourceFilter = true;
		}
		return changedDataSourceFilter;
	};

	/*
	 * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
	 * ou produtos correspondentes à máscara utilizada. Através de cada nível
	 * da máscara, é gerado o número correspondente à ele.
	 *
	 * @param string maskName
	 * @param int firstLevel
	 *
	 * @return array
	 *
	 * Exemplo:
	 * General.getLevelByMask('CDPICTPRAT', 1);
	 */
	this.getLevelByMask = function (maskName, firstLevel) {

		firstLevel = toNumber(firstLevel);

		var mask = getLocalVar(maskName);
		var splitMask = mask.split('.');
		var biggestLevel = splitMask.length;

		var level = [];
		var objLevel = {};

		for (var i = 0; i < biggestLevel; i++) {

			objLevel = {
				nivel: i + firstLevel,
				caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
			};

			objLevel.nivel = objLevel.nivel.toString();
			objLevel.caracteres = objLevel.caracteres.toString();

			level.push(objLevel);
		}

		return level;
	};

	var toNumber = function (value) {

		if (!value && value !== 0) {

			throw 'firstLevel is not defined';

		} else if (typeof value !== 'number' && value) {

			value = parseInt(value);

			if (isNaN(parseInt(value))) {

				throw 'Invalid firstLevel';

			} else {

				return value;
			}
		}
		return value;
	};

	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
		templateManager.updateTemplate();
	};

	this.setLoggedUnit = function (widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
	};

	var currentIdgrupoper;
	var currentCallback;
	var currentParameters;
	var gruposDeOperador = {
		LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
	};
	var fields = [
		{
			"name": "email",
			"label": "E-mail",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"template": "field/text-edit.html",
			"events": [],
			"fieldGroup": "AUTH",
			"validations": {
				"required": true,
				"mail": true
			}
        },
		{
			"name": "senha",
			"label": "Senha",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"fieldGroup": "AUTH",
			"template": "field/password-edit.html",
			"events": [],
			"validations": {
				"required": true
			}
        }
    ];
	var actions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.validateUser(args.owner.widget);"
                }
            ]
        }
    ];

	this.openAuthPopup = function (idgrupoper, message, callback, parameters) {

		if (!idgrupoper) {
			throw 'idgrupoper is not defined';
		}

		initAuthVariables(idgrupoper, callback, parameters);

		if (!message) {
			message = gruposDeOperador[currentIdgrupoper] || '';
		}

		var widgetAuth = metaDataFactory.widgetFactory({
			"id": 0,
			"name": "authWidget",
			"label": "Autenticação",
			"template": "widget/form.html",
			"container": templateManager.container,
			"isVisible": true,
			"popupNoBlock": false,
			"fieldGroups": [
				{
					"id": 100000,
					"name": "AUTH",
					"opened": true,
					"collapsible": false,
					"label": message
                }
            ],
			"fields": fields,
			"actions": actions,
			"events": []
		});

		ScreenService.openPopup(widgetAuth);

	};

	var initAuthVariables = function (idgrupoper, callback, parameters) {
		currentIdgrupoper = idgrupoper;
		currentCallback = callback || null;
		currentParameters = parameters || [];
	};

	this.validateUser = function (widget) {

		var self = this;

		if (widget.isValid()) {

			var email = widget.currentRow.email;
			var senha = widget.currentRow.senha;

			autenticar(email, senha).then(function (callback) {

				getPermission(callback.dataset.user.cdoperador).then(function (permission) {

					if (permission) {

						if (typeof currentCallback == 'function') {

							ScreenService.closePopup();

							currentCallback.apply(null, currentParameters);

						} else {

							throw 'callback is not a function';

						}

					} else {

						MessageService.showMessageByCode('lib#GEN_INVALID_OPERGROUP', null, { type: 'ERROR' } );

					}

				}, function (callback) {

					this.callbackError(callback);

				}.bind(self));

			}, function (callback) {

				this.callbackError(callback);

			}.bind(self));
		}

	};

	var autenticar = function (email, senha) {

		return restEngine.request('/Autenticar', {
			"requestType": "FilterData",
			"filter": [
				{
					name: "email",
					operator: "=",
					value: email
                },
				{
					name: "senha",
					operator: "=",
					value: senha
                }
            ]
		});

	};

	var getPermission = function (cdoperador) {

		return restEngine.request('/validateUserGroup', {
			"requestType": "Row",
			"row": {
				"idgrupoper": currentIdgrupoper,
				"cdoperador": cdoperador
			}
		}).then(function (callback) {

			return callback.dataset.permission.isValid;
		});

	};

	this.callbackError = function (callback) {

		if (callback.data) {

			ScreenService.showMessage(ScreenService.i18n(callback.data.error));

		} else {
			
			MessageService.showMessageByCode('lib#GEN_TIMEOUT', null, { type: 'ERROR' } );
		}
	};

	this.comparaPratoProduto = function (pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var menornv;
		if (nvinicial > nvfinal) {
			menornv = nvfinal;
		} else {
			menornv = nvinicial;
		}

		var resultado = true;
		var i = 0;

		while (pinicial[i] == pfinal[i] && i < menornv) {
			i++;
		}

		if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
			resultado = false;
		} else {
			resultado = true;
		}

		return resultado;
	};

	var reportWidget = {
		"id": 0,
		"name": "reportWidget",
		"label": "Tipo de Relatório",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};
	var reportFields = [
		{
			"id": 99999999,
			"name": "QR2TYPE",
			"label": "",
			"class": 12,
			"readOnly": false,
			"isVisible": true,
			"isVisibleForm": true,
			"isVisibleGrid": false,
			"template": "field/radio-group.html",
			"events": [],
			"valueField": "value",
			"descriptionField": "name",
			"defaultOption": 0,
			"dataSource": {
				"data": [
					{
						name: "pdf",
						value: "PDF"
					},
					{
						name: "xls",
						value: "XLS"
					},
					{
						name: "rtf",
						value: "RTF"
					},
					{
						name: "txt",
						value: "TXT"
					}
				]
			}
        }
    ];
	var reportActions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.generateReport(args.owner.widget, args);"
                }
            ]
        }
	];
	
	var reportCallback;
	var reportParameters;
	var reportWidgetParentName;

	this.openReportPopup = function (widget, callback, parameters, reportFormats) {

		reportCallback = callback;
		reportParameters = parameters || [];
		reportWidgetParentName = widget.name;
		reportWidget.container = widget.container;
		reportWidget.fields = reportFields;
		reportWidget.actions = reportActions;

		var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

		updateReportWidgetFormats(reportWidgetInitialized, reportFormats);

		ScreenService.openPopup(reportWidgetInitialized);
	};

	var updateReportWidgetFormats = function(reportWidget, reportFormats) {
		if (Array.isArray(reportFormats) && reportFormats.length) {
			var formatField = reportWidget.getField("QR2TYPE");
			formatField.dataSource.data = reportFormats;
		}
	};

	this.generateReport = function (widget, args) {

		var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

		reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

		args.row = reportWidgetParentCurrentRow;

		if (typeof reportCallback == 'function') {

			reportCallback.apply(null, reportParameters);
		}

		ScreenService.closePopup();

	};

	this.saveRow = function (args, closeSwipe) {
		args.owner.widget.saveRow().then(function () {
			var currentRow = args.owner.widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			args
				.owner
				.widget
				.reload();
			if (closeSwipe) {
				ScreenService.closeSwipe();
			}
		});
	};

	this.openSelectionOnce = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		if (!filterWidget.wasOpened) {
			filterWidget.edit();
			ScreenService.openPopup(filterWidget);
			filterWidget.wasOpened = true;
		}
	};

	this.openSelection = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		filterWidget.edit();
		ScreenService.openPopup(filterWidget);
	};

	this.getContainerName = function () {
		return templateManager.container.name;
	};

	this.backOrCancel = function (args) {
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(true);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}
		ScreenService.goBack();
		return deferred.promise;
	};

	this.edit = function (widget) {
		widget.backupRow = Object.assign({}, widget.currentRow);
		widget.edit();
	};

	this.modEdit = function (widget) {
		parent.edit(widget);
		var widgetParent = widget.widgets;
		if (widgetParent.length)
			parent.setVisibleforTabbed(widgetParent, false, widget);
		// widget.template = widget.formTemplate = widget.templateToForm;
	};

	this.cancelModEdit = function (args, widgetParent) {
		parent.backOrCancel(args).then(function (success) {
			var widget = args.owner.widget;
			var widgetParent = widget.widgets;
			if (widgetParent.length)
				parent.setVisibleforTabbed(widgetParent, true);
			// widget.template = widget.formTemplate = widget.templateToView;
		});
	};

	this.setVisibleforTabbed = function (widgets, isVisible, exception) {
		widgets.forEach(function (widget) {
			if (exception) {
				if (exception.name != widget.name)
					widget.isVisible = isVisible;
			} else {
				widget.isVisible = isVisible;
			}
		});
	};

	this.setContextVar = function (name, containerName, value) {
		localStorageEngine.setLocalVar(containerName + '_' + name, value);
	};

	this.getContextVar = function (name, containerName) {
		return localStorageEngine.getLocalVar(containerName + '_' + name);
	};

	var _getWidget = function (widgetName) {
		var isPopup = arguments[1] || null;
		if (!!templateManager.container) {
			var widget;
			if (!isPopup) {
				widget = templateManager.container.getWidget(widgetName);
			} else {
				widget = templateManager.container.getWidget(widgetName);
				if(widget)
					if(widget.hasOwnProperty('__filterPopup')) 
						widget = widget.__filterPopup;
					else{
						widget = widget.appliedFilter;
					}
			}
			if (widget) {
				return widget;
			}
		}
		return null;
	};

	var _getField = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var field = widget.getField(name);
		if (field) {
			return field;
		}
		return null;
	};

	var _getAction = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var action = widget.getAction(name);
		if (action) {
			return action;
		}
		return null;
	};

	this.getFieldValue = function (name, widgetName) {
		var isPopup = arguments[2] || null;
		var widget = _getWidget(widgetName, isPopup);
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		} else {
			return "";
		}
	};

	this.setEnableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = false;
	};

	this.setDisableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = true;
	};

	this.setVisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = true;
	};

	this.setInvisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = false;
	};

	this.setEnableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = false;
	};

	this.setDisableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = true;
	};

	this.setVisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = true;
	};

	this.setInvisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = false;
	};

	this.now = function () {
		return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
	};

	this.today = function () {
		return moment(new Date()).format('DD/MM/YYYY');
	};

	this.openReport = function (route, reportName, params) {
		params.__report_name = reportName;
		
		var lastActiveWidget = templateManager.container.getActiveWidget().lastActiveWidget;
	
		if (lastActiveWidget && lastActiveWidget.beforeOpenReport) {
		    lastActiveWidget.beforeOpenReport({
		        reportParams: params
		    });
		}
		
		return restEngine.post(route, {
			"requestType": "Row",
			"row": params
		}, null, null, null, null, 3000000).then(function (response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				var reportOpened = window.open(report.message);
				if(!reportOpened){
					MessageService.showMessageByCode('lib#GEN_POPUP_BLOCKER', null, { type: 'ERROR' } );
				}
			}
		}).catch(function (response) {
			if (response.data && response.data.error) {
				ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
			}
		});
	};

	// CRUD
	/**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function (widget, afterValid) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
					widget.reload().then(function () {
						deferred.resolve(success);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function (widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
					" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		} catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function (arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			} else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.deleteCrud = function (widget) {
		var deferred = ZHPromise.defer();
		MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};

    this.deleteCrudCheckedRows = function (widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

	this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}

	function preventEvent(e) {
		e.preventDefault();
	}

	function padLeft(value, length) {
		return (value.toString().length < length) ? padLeft("0" + value, length) : value;
	}

	function padRight(value, length) {
		return (value.toString().length < length) ? padRight(value + "0", length) : value;
	}

	this.getCurrentDate = function (format) {
		return moment().format(format);
	};

	this.setDateAttrToVarcharOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
	};

	this.setAttrOnCurrentRow = function (currentRow, attrName, value) {
		currentRow[attrName] = value;
	};

	this.setFloatAttrOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
	};

	this.valueToFloat = function (value) {
		if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
			if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
				return parseFloat(value);
			} else if (value.indexOf(',') > value.indexOf('.')) {
				return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
			} else if (value.indexOf(',') < value.indexOf('.')) {
				return parseFloat(value.replace(/\,/g, ''));
			}
		} else {
			return value;
		}
	};

	this.toLocalDate = function (element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	this.toLocalDateFormat = function (value, params) {
		if (value) {
			var valueDate = moment(value, params.format);
			value = valueDate.format(params.toFormat);
		}

		return value;
	};

	maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

	var maskFunction = function (element, viewValue, params, callBack) {
		// do something
	};

	var maskFormatFunction = function (value, params) {
		if (value) {
			var length = value.length;
			if (length === 11)
				value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
			else if (length === 14)
				value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
			return value;
		}
	};

	maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

	/*
	 * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
	 * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
	 * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
	 */

	var constAllWidgets;

	this.setVisibilityByProjectID = function (widgets) {
		if (widgets.length > 1) {
			widgets.forEach(function (childWidget) {
				parent.setVisibilityByProjectID(childWidget.widgets);
				if (childWidget.projects) {
					childWidget.projects.forEach(function (proj) {
						childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
					});
				}
			});

		}

	};

	this.validateFilterPopup = function (args) {
		if(args.owner.hasOwnProperty('__filterPopup')){
			var filterPopup = args.owner.__filterPopup;
		
		if (!filterPopup.isValid())
			args.e.preventDefault();
		}	
	};

	var timeFormats = {
		"hh": isValidHour,
		"mm": isValidMinutesOrSeconds,
		"ss": isValidMinutesOrSeconds,
		"hh:mm": isValidHourAndMinutes,
		"mm:ss": isValidMinutesAndSeconds,
		"hh:mm:ss": isValidTime,
	};

	function isValidHour(hour) {
		if (parseInt(hour) > 23 || parseInt(hour) < 0)
			return false;
		else
			return true;
	}

	function isValidMinutesOrSeconds(minuteOrSecond) {
		if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
			return false;
		else
			return true;
	}

	function isValidHourAndMinutes(hourAndMinute) {
		if (hourAndMinute) {
			var timeArray = hourAndMinute.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidMinutesAndSeconds(minutesAndSeconds) {
		if (minutesAndSeconds) {
			var timeArray = minutesAndSeconds.split(':');
			return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidTime(time) {
		if (time) {
			var timeArray = time.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
		}
		return true;
	}

	var validTime = function (value, format) {
		var validFunction;
		if (!format)
			throw 'format is not defined';
		if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
			validFunction = timeFormats[format.toLowerCase()];
		else
			throw 'time format is not registered';
		return {
			"valid": validFunction(value),
			"message": ScreenService.i18n("Não é um tempo válido.")
		};
	};

	this.registerTimeFormat = function (format, formatFunction) {
		if (typeof format !== 'string')
			throw 'time format must be an string';
		if (timeFormats.hasOwnProperty(format.toLowerCase()))
			throw 'time format already registered';
		timeFormats[format] = formatFunction;
	};

	ValidationEngine.registerValidation('isValidTime', validTime);

	/*
	 * Função que preenche um valor com um único caractere até um certo tamanho.
	 * Faz zerofill pela esquerda por default.
	 *
	 * @param string value
	 * @param int length
	 * @param boolean toRight
	 * @param char filler
	 *
	 * @return string
	 *
	 * Exemplos:
	 * General.fill('123', 10);            //'0000000123'
	 * General.fill('123', 10, true);      //'1230000000'
	 * General.fill('123', 10, null, '_'); //'_______123'
	 */
	this.fill = function (value, length, toRight, filler) {
		if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight)
				while (value.length + filler.length <= length) value = value + filler;
			else
				while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
	};

	/*
	 * Função que faz a validação de CNPJ.
	 *
	 * @param string CNPJ
	 *
	 * @return boolean
	 */
	function validCNPJ(CNPJ) {
		if (CNPJ === undefined || CNPJ === null)
			return false;

		CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

		if (CNPJ.length != 14)
			return false;

		if (CNPJ == "00000000000000" ||
			CNPJ == "11111111111111" ||
			CNPJ == "22222222222222" ||
			CNPJ == "33333333333333" ||
			CNPJ == "44444444444444" ||
			CNPJ == "55555555555555" ||
			CNPJ == "66666666666666" ||
			CNPJ == "77777777777777" ||
			CNPJ == "88888888888888" ||
			CNPJ == "99999999999999")
			return false;

		var tamanho = CNPJ.length - 2;
		var numeros = CNPJ.substring(0, tamanho);
		var digitos = CNPJ.substring(tamanho);
		var soma = 0;
		var pos = tamanho - 7;

		for (var i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(0)) //debugger;
			return false; //'resultado' é o 1º digito de validação do CNPJ

		tamanho = tamanho + 1;
		numeros = CNPJ.substring(0, tamanho);
		soma = 0;
		pos = tamanho - 7;

		for (i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(1)) //debugger;
			return false; //'resultado' é o 2º digito de validação do CNPJ

		return true;
	}

	this.validateCNPJ = validCNPJ;

	/*
	 * Função que faz a validação de CPF.
	 *
	 * @param string CPF
	 *
	 * @return boolean
	 */
	function validCPF(CPF) {
		if (CPF === undefined || CPF === null)
			return false;

		CPF = CPF.toString().replace(/[\D]+/g, '');

		if (CPF.length != 11)
			return false;

		if (CPF == "00000000000" ||
			CPF == "11111111111" ||
			CPF == "22222222222" ||
			CPF == "33333333333" ||
			CPF == "44444444444" ||
			CPF == "55555555555" ||
			CPF == "66666666666" ||
			CPF == "77777777777" ||
			CPF == "88888888888" ||
			CPF == "99999999999")
			return false;

		var soma = 0;
		var resto;

		for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(9, 10))) //debugger;
			return false; //'resto' é o 1º dígito de validação do CPF

		soma = 0;
		for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(10, 11))) //debugger;
			return false; //'resto' é o 2º dígito de validação do CPF

		return true;
	}

	this.validateCPF = validCPF;

	/*
	 * Função que faz a formatação de uma inscrição,
	 * seguindo o padrão de código genérico, CPF, CNPJ e
	 * inscrição genérica. Caso o parâmetro "force" for
	 * true, o método tenta forçar uma inscrição válida.
	 *
	 * Retorna um objeto com as seguintes propriedades:
	 *  * parameter //string de entrada contendo apenas números
	 *  * value     //string formatada conforme o tipo da inscrição
	 *  * tipo.cod  //char do código padrão do tipo da inscrição
	 *  * tipo.desc //string de descrição do tipo da inscrição
	 *  * valid     //boolean
	 *
	 * @param string inscricao
	 * @param boolean force
	 *
	 * @return object
	 */
	this.inscricaoFormat = function (inscricao, force) {
		if (inscricao === undefined || inscricao === null || inscricao === '')
			return {
				'parameter': '',
				'value': '',
				'tipo': {
					'cod': null,
					'desc': null
				},
				'valid': false
			};

		if (force === undefined || force === null)
			force = false;
		inscricao = inscricao.toString().replace(/[\D]+/g, '');

		var obj = {};
		obj.tipo = {};

		if (inscricao.length <= 5) {
			obj.tipo.cod = 'O';
			obj.tipo.desc = 'Código Genérico';
			obj.valid = true;
			obj.value = this.fill(inscricao, 5);
		} else {
			if (inscricao.length == 11) {
				obj.tipo.cod = 'F';
				obj.tipo.desc = 'Pessoa Física';
				obj.valid = this.validateCPF(inscricao);
				obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
			} else if (inscricao.length == 14) {
				obj.tipo.cod = 'J';
				obj.tipo.desc = 'Pessoa Jurídica';
				obj.valid = this.validateCNPJ(inscricao);
				obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
			} else force = true;

			if (force && obj.valid !== true) {
				obj.tipo.cod = 'I';
				obj.tipo.desc = 'Inscrição Internacional';

				if (inscricao.length <= 20) {
					obj.valid = true;
					obj.value = this.fill(inscricao, 20);
				} else {
					obj.valid = false;
					obj.value = inscricao.substr(0, 20);
				}
			}
		}

		obj.parameter = inscricao;
		return obj;
	};

	var CPFValidation = function (value, validationAttrs) {
		return {
			"valid": validCPF(value),
			"message": ScreenService.i18n('CPF Inválido')
		};
	};

	ValidationEngine.registerValidation('CPF', CPFValidation);

	var CNPJValidation = function (value, validationAttrs) {
		return {
			"valid": validCNPJ(value),
			"message": ScreenService.i18n('CNPJ Inválido')
		};
	};

	ValidationEngine.registerValidation('CNPJ', CNPJValidation);

	this.callRestEngine = function (dataSource, requestType, requestValue, disableLoader) {
		var defer = ZHPromise.defer();
		var params = {};
		
		switch (requestType) {
		case "Row":
			params = {
				"requestType": "Row",
				"row": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "DataSet":
			params = {
				"requestType": "DataSet",
				"dataset": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "FilterData":
			params = {
				"requestType": "FilterData",
				"filter": requestValue,
				"disableLoader": disableLoader
			};
			break;

		default:
			return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
		}

		return restEngine.request(dataSource, params);
	};

	/*
	 * Função que muda a acessibilidade de uma ou mais
	 * widgets.
	 *
	 * @param array(objects) widgets
	 * @param boolean enabled
	 *
	 * Exemplos:
	 * General.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
	 * General.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
	 */
	this.changeWidgetAcessibility = function (widgets, enabled) {
		widgets.forEach(function (widget) {
			widget.enabled = enabled;
		});

		templateManager.updateTemplate();
	};

	/*
	 * Função que altera o template de um form
	 * entre edição e visualização. Retorna um boolean
	 * que indica se a widget era do tipo form e a
	 * alteração foi realizada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.formWidgetSwitchTemplate = function (widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	/*
	 * Função que coloca uma widget em modo de edição,
	 * bloqueando as demais widgets. Retorna um boolean
	 * que indica se a alteração de modo foi executada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.widgetEdit = function (widget) {
		if (!widget.editing) {
			//Trata a edição de uma widget filha do tipo form
			if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
				var widgetsToDeactivate = widget.parent.widgets;

				this.changeWidgetAcessibility(
					widgetsToDeactivate.filter(function (subwidget) {
						return subwidget.name != widget.name;
					}).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
					false);
			} else //trata a edição dos demais casos
				this.changeWidgetAcessibility(widget.widgets, false);

			//propriedades a serem ignoradas na criação do backup
			var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

			//faz backup da currentRow para permitir a recuperação dos dados
			widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
			widget.edit();
			return true;
		} else return false;
	};

	/*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * própria widget. Provavelmente usado na action de
	 * 'voltar' do back_view.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToList = function (widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * widget parent. Provavelmente usado na action de
	 * 'voltar' do back_list.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToParent = function (widget) {
		ScreenService.closeSwipe().then(function () {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	/*
	 * Procedimento que habilita inserção em uma widget,
	 * bloqueando as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetNewRow = function (widget) {
		this.changeWidgetAcessibility(widget.widgets, false);

		widget.newRow();
	};

	/*
	 * Função que salva os dados de uma widget e retorna
	 * para o modo adequado de visualização ou listagem.
	 * Faz exibição de mensagens de aviso para o usuário
	 * conforme padronização e desbloqueia as demais
	 * widgets.
	 *
	 * Basicamente, um saveCrud adaptado ao padrão.
	 *
	 * @param object widget
	 *
	 * @return promise
	 */
	this.widgetSave = function (widget) {
		var deferred = ZHPromise.defer();

		if (widget.isValid()) {
			widget.currentRow.changed = true;

			if (!widget.currentRow.__is_new)
				widget.currentRow.__is_new = false;
			else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = widget.paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                	widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
			}

			widget.saveRow().then(function (response) {
				widget.currentRow.changed = false;
				delete widget.backupRow;

				if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
					widget.view();

					var widgets = widget.parent.widgets;
					this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
				} else {
					if (widget.currentRow.__is_new) {
						ScreenService.closeSwipe();
						widget.list();
					} else widget.view();

					this.changeWidgetAcessibility(widget.widgets, true);
				}

				if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
				else widget.reload();

				deferred.resolve(response);
			}.bind(this), function (error) {
				var errorMessage = 'Timeout';

				if (error.data) errorMessage = error.data.error;

				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
	};

	/*
	 * Função para validar containers abertos a partir de outros containers.
	 * Se o container atual não foi aberto a partir do container pai, a
	 * callback é executada. Se nenhuma callback for passada nos parâmetros,
	 * o container pai é aberto.
	 *
	 * @param: object {parentContainerName, callback, scope, params}
	 *
	 * Exemplos:
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
	 */
	this.checkParentContainer = function (containerProperties) {
		containers = templateManager.containers;
		if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
			if (angular.isFunction(containerProperties.callback)) {
				containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
			} else {
				ScreenService.openWindow(containerProperties.parentContainerName);
			}
		}
	};

	var messagesWidget = {
		"id": 0,
		"name": "messagesWidget",
		"label": "Mensagens",
		"messagesDataset": "",
		"template": "widget/form.html",
		"floatingControl": {
			"customizationAction": {
				"filter": false,
				"view": false,
				"layout": false
			}
		},
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};

	var messagesFields = [{
		"label": "",
		"name": "CONTENT",
		"readOnly": true,
		"template": "field/textarea.html",
		"textRows": 14,
		"isVisible": true,
		"class": 12,
		"descriptionOrder": true,
		"showOnView": true,
		"showOnForm": true,
		"showOnList": true
    }];

	var messagesActions = [{
		"name": "cancelar",
		"label": "Cancelar",
		"idVisible": true,
		"hideIcon": true,
		"showAsAction": "back",
		"events": [{
			"name": "ActionEvent",
			"code": "ScreenService.closePopup();"
        }]
    }, {
		"name": "exportar",
		"label": "Exportar",
		"idVisible": true,
		"icon": "export",
		"showAsAction": "always",
		"events": [{
			"name": "ActionEvent",
			"code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
        }]
    }];

	/* Função abre um popup com um Text Area a título de mostrar erros em lista.
	 * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
	 * @params: widget, dataset, popupName
	 */
	this.showErrorMessages = function (widget, dataset, popupName) {
		messagesWidget.container = widget.container;
		messagesWidget.fields = messagesFields;
		messagesWidget.actions = messagesActions;

		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

	/* Gera arquivo básico baseado em linhas de dataSet.
	 * @params: dataset, object {fileName}
	 * !!! fileExtension: por enquanto somente TXT;
	 * Exemplo:
	 * General.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
	 */

	this.generateFileByDataSet = function (dataset, fileProperties) {
		var content = "";
		var name = 'content';
		var extension = 'text/plain';

		if (fileProperties !== undefined) {
			if (fileProperties.fileName !== undefined)
				name = fileProperties.fileName;
		}

		dataset.forEach(function (line) {
			content += line + "\r\n";
		});

		var element = document.createElement('a');
		element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
		element.setAttribute('download', name);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

	/**
	 * Este método verifica se a widget foi filtrada usando o zh-grid-filter
	 * @param Widget widget
	 * @return boolean
	 */
	this.isWidgetFiltered = function (widget) {
		var isFiltered = true;
		var currentFilter;
		if(widget.hasOwnProperty('__filterPopup'))
			currentFilter = widget.__filterPopup.currentFilter;
		else
			currentFilter = widget.appliedFilter.FILTER_ROW;
			
		var requiredFields = widget.fields.filter(function (field){
			return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
		});
		if(requiredFields.length && Array.isArray(currentFilter)) {
			isFiltered = false;
		}
		return isFiltered;
	};

	function isRequiredFieldInFilter(field) {
		return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
	}

	function isRequiredFieldWithoutFilterProperties(field) {
		return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
	}

	/**
	 * Este método valida a inserção de registros na widget
	 * @param Widget widget
	 */
	this.newRow = function (widget) {
		if(this.isWidgetFiltered(widget)) {
			widget.newRow();
		} else {
			MessageService.showMessageByCode('lib#GEN_NOT_FILTER', null, { type: 'ERROR' } );
		}
	};

    /*
     * Função que adiciona à currentRow o(s) valor(es)
     * da widget parent necessários para salvar o registro.
     *
     * @param object widget
     * @param object parents
     *
     * @return promise
     */

    this.saveWithParent = function(widget, parents) {
        if(Array.isArray(parents)) {
            parents.forEach(function(parent){
                widget.currentRow[parent] = widget.parent.currentRow[parent];
            });
            return this.widgetSave(widget);
        }
        else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
    };
    
    /*
     * Aceita [- . , 0 1 2 3 4 5 6 7 8 9]
     *
     */
    
     this.justDigits = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonDigits;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justDigits(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonDigits(e) {
        var digits = [44, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var digitCode = e.which;
        if (digits.indexOf(digitCode) < 0) {
            preventEvent(e);
        }
    }

    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousands, precision) {
    	precision = precision || 2;
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousands": thousands,
            "precision": precision
        };
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    /**
     * Função responsável por aplicar uma máscara do type currency
     * no field.
     * 
     * @param {String} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.' || value[0] === ',')
                value = '0' + value;
            if (params.decimal === ',')
                value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            var counter = angular.copy(int.length / 3);

            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');
            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('maskCurrency', maskFunction, this.maskCurrency);
    
     /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} value - Valor numérico.
     */

    this.parseFloat = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/\,/g.test(value))
            value = value.toString().replace(',', '.');
        return parseFloat(value);
    };
    
    /**
     * Função que formata números float vindos do banco, ou em
     * padrão americano e passa para o formato brasileiro.
     * 
     * @param {String} value - Valor numérico.
     */
    
    var numberFormatBr = function(value) {
        if (/^(.*)(\,+).*(\.+)(.*)$/gm.test(value)) {
            var loop = value.toString().split(',');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace(',', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
		else if(/^(\d*)([\.]*)((\d|\.)*)$/.test(value)){
			value = value.split('.');
			value[value.length-1] = ','+value[value.length-1];
			value = value.join('');
		}
        return value.toString();
    };
    
    /**
     * Função criada para corrigir o erro da maxlength.
     * Em vez do maxlength deve ser usada a propriedade maxLengthCurrency,
     * que considera apenas os números inteiros como length.
     * 
     * Função deve ser usada no evento FieldOnKeydown.
     * 
     * @param {Args} args - objeto args.
     * @param {Field} field - field com a propriedade maxLengthCurrency.
     */
    
    this.lengthCurrency = function(args, field) {
        var value;
        var fieldValue;
        
        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = field.widget.currentRow[field.name + '_inline'];
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        } else {
            fieldValue = field.value();
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        }
    
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        
        if (realLength >= field.maxLengthCurrency && !/^(\d+)(\,){1}(\d*)$/gm.test(value)) {
            var allowedKeys = [8, 37, 38, 39, 40, 46, 110, 188, 190, 194];
            if (!allowedKeys.includes(args.e.keyCode))
                args.e.preventDefault();
        }
    };
    
    var treatResponse = function(response, Authenticator) { //
		var url = response.responseURL;
	    var responseURL = null;
	    var responseData = response.response;
	    
	    if (url.indexOf('index.php/') > -1) {
	        responseURL = url.slice(url.indexOf('index.php/') + 9);
	    }
	    
		if(responseURL){
			var responseObj = JSON.parse(responseData);
	        var errorCode = responseObj.error;
			if(Authenticator){
				Authenticator.run(responseURL, responseObj, errorCode);
			} else if(errorCode && errorCode.indexOf('AUTH_') > -1){// Exception: AUTH_INVALID_UNIT 
	            var cleanError = parent.getCleanError(errorCode);
				MessageService.showMessageByCode(cleanError);
			}
		}
	};
	
	this.getCleanError = function(msg){
		return msg.match(/AUTH_[^\s]+/g)[0];
	};
	
	/**
	 * Overwrite XMLHttpRequest.prototype.open
	 * and add a listener on it.
	 */
	this.initRoutesListener = function(Authenticator) {
	    (function(open) {
	        XMLHttpRequest.prototype.open = function(method, url, async, user, pass) {
	            this.addEventListener("readystatechange", function() {
	            	try{
	                	treatResponse(this, Authenticator);
	            	} catch(e){
	            		return false;
	            	}
	            }, false);
	
	            open.call(this, method, url, async, user, pass);
	        };
	    })(XMLHttpRequest.prototype.open);
	};
	
	this.isValidFiliaisValuesCP = function(values){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
        if(arrFilialValida && cacheLoaded){ 
        	var arrValues = [];
        	Array.isArray(values)? arrValues = values : arrValues.push(values);
        	var isInvalid = arrValues.some(function(value){return !arrFilialValida.includes(value)}.bind(this));
        	return !isInvalid;
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then( function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.isValidFiliaisValuesCP(values);
        		}
        	}.bind(this));
        }
	};
	
	this.validateFieldFiliaisCP = function(field){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
		var filialValues;
        if(arrFilialValida && cacheLoaded){ 
            if(field.name == 'CDFILIAL')
                filialValues = field.value();
            else{
                if(field.outData.CDFILIAL){
                    filialValues = field.widget.currentRow[field.outData.CDFILIAL];
                }else{
                    return console.log('Não existe o campo CDFILIAL neste FIELD, usar a função validateFieldFiliaisCP(field)');   
                }
                if(filialValues && filialValues.length > 0){
                	var arrValues = [];
		        	Array.isArray(filialValues)? arrValues = filialValues : arrValues.push(filialValues);
		            var filtered = arrValues.filter(function(filial){return arrFilialValida.includes(filial)}.bind(this));
		            var invalids = arrValues.filter(function(filial){return !arrFilialValida.includes(filial)}.bind(this));
		            
		            if(filtered.length == 0 || !this.isValidFiliaisValuesCP(arrValues)){
		            	MessageService.showMessageByCode('lib#CP_BRANCH_NOT_ALLOWED', null, { type: 'ERROR' });
		            	var payload = {CDFILIAL:invalids};
		            	this.callRestEngine('lib#/lib_saveOrgFilial', 'DataSet', [payload]);
		            	/*
		            	Descomentar essa parte quando for para bloquear totalmente o uso de unidades nao cadastradas
		            	
		            	field.setValue('');
		            	field.widget.currentRow[field.name] = '';
			            //remove valores nao liberados do datasource.data
			            var filteredDataSet = field.dataSource.data.filter(function(dataSetRow){return arrFilialValida.some(function(row){return row == dataSetRow.CDFILIAL}.bind(this))}.bind(this));
			            field.dataSource.data = filteredDataSet;
			            
			            //Remove valores nao liberados do spam do autocomplete.
			            if(field.template == 'zh-select-autocomplete#field/select-autocomplete.html'){
				            var spamList = field.element.context.parentElement.getElementsByClassName('list-items')[0].children;
				            for (var i = 0; i < spamList.length -1; i++) {
				            	if(spamList[i].className != 'view-more' && !arrFilialValida.some(function(row){return spamList[i].getAttribute('data-item-identifier').indexOf(row) >= 0}.bind(this))){
							    	spamList[i].remove();
				            		i--
				            	}
							}
			            }//fim if
			            */
		            }
                }
            }
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then(function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.validateFieldFiliaisCP(field);
        		}
        	}.bind(this));
        }
	};
	
	this.setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            if(Array.isArray(outDataValue[key])){
                var newArray = [];
                outDataValue[key].forEach(function(value){ newArray[value] = value}.bind(this)); // jshint ignore:line
                this.setCurrentRowWithOutData(currentRow, valuesToSetOnCurrentRow, newArray);
            }
            else
                currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };
	
};

Configuration(function (ContextRegister) {
	ContextRegister.register('General', General);
});